# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for start game window related functionality
"""
from herculeum.ui.gui.widgets import AnimatedLabel, TimerAdapter
from PyQt4.QtCore import Qt, QTimer
from PyQt4.QtGui import QDialog, QHBoxLayout, QLabel, QSizePolicy, QVBoxLayout


class StartGameWidget(QDialog):
    """
    Start menu

    .. versionadded:: 0.5
    """

    def __init__(self, generator, config,
                 parent, application, surface_manager, flags):
        """
        Default constructor
        """
        super().__init__(parent, flags)

        self.application = application
        self.surface_manager = surface_manager
        self.config = config

        self.generator = generator

        self.player_character = None
        self.class_description = None
        self.class_name = None
        self.class_icon = None
        self.selected_index = None
        self.class_names = sorted(application.config.player_classes)
        self.class_config = application.config.player_classes

        self.animation_adapter = None
        self.animation_timer = None

        cnf = self.config
        self.konami = [cnf.move_up, cnf.move_up,
                       cnf.move_down, cnf.move_down,
                       cnf.move_left, cnf.move_right,
                       cnf.move_left, cnf.move_right,
                       cnf.action_b, cnf.action_a]
        self.konami_index = 0

        self.__set_layout()

    def __set_layout(self):
        """
        Set layout of this widget
        """
        self.setSizePolicy(QSizePolicy(
                                       QSizePolicy.Fixed,
                                       QSizePolicy.Fixed))

        main_layout = QVBoxLayout()
        top_layout = QHBoxLayout()
        middle_layout = QHBoxLayout()
        bottom_layout = QHBoxLayout()

        main_layout.addLayout(top_layout)
        main_layout.addLayout(middle_layout)
        main_layout.addLayout(bottom_layout)

        self.class_name = QLabel('')
        self.class_name.setMinimumSize(200, 50)
        self.class_name.setMaximumSize(200, 50)
        self.class_name.setAlignment(Qt.AlignCenter)

        self.animation_adapter = TimerAdapter()
        self.animation_timer = QTimer(self)
        self.animation_timer.timeout.connect(self.animation_adapter.trigger_animations)
        self.animation_timer.start(500)

        self.class_icon = AnimatedLabel(self.animation_adapter)
        self.class_icon.setMinimumSize(64, 64)
        self.class_icon.setMaximumSize(64, 64)
        self.class_icon.setAlignment(Qt.AlignCenter)
        self.class_description = QLabel('')
        self.class_description.setWordWrap(True)
        self.class_description.setMinimumSize(600, 300)
        self.class_description.setMaximumSize(600, 300)
        self.class_description.setAlignment(Qt.AlignTop)

        top_layout.addStretch()
        top_layout.addWidget(self.class_name)
        top_layout.addStretch()
        middle_layout.addStretch()
        middle_layout.addWidget(self.class_icon)
        middle_layout.addStretch()
        bottom_layout.addStretch()
        bottom_layout.addWidget(self.class_description)
        bottom_layout.addStretch()

        self.setLayout(main_layout)

        self.selected_index = 0
        if len(self.class_names) > 0:
            self._show_character(
                    self.class_config[self.class_names[self.selected_index]])

    def _show_character(self, character):
        """
        Show character on screen

        .. versionadded:: 0.8
        """
        icon = self.surface_manager.get_icon(character['\ufdd0:icons'])

        self.class_icon.setPixmap(icon)
        self.class_name.setText(character['\ufdd0:name'])
        self.class_description.setText(character['\ufdd0:description'])

    def __generate_character(self):
        """
        Generate player character based on selected settings
        """
        self.player_character = None
        self.player_character = self.generator(self.class_names[self.selected_index])

    def keyPressEvent(self, event):
        """
        Process keyboard events
        """
        if event.key() in self.konami[self.konami_index]:
            self.konami_index = self.konami_index + 1
        else:
            self.konami_index = 0

        if self.konami_index == len(self.konami):
            self.application.enable_cheat()

        if event.key() in self.config.action_a:
            self.__generate_character()
            self.animation_adapter.glyphs.clear()
            self.accept()
        elif event.key() in self.config.move_right:
            self.selected_index = self.selected_index + 1
            if self.selected_index >= len(self.class_names):
                self.selected_index = 0
            self._show_character(
                    self.class_config[self.class_names[self.selected_index]])
        elif event.key() in self.config.move_left:
            self.selected_index = self.selected_index - 1
            if self.selected_index < 0:
                self.selected_index = len(self.class_names) - 1
            self._show_character(
                    self.class_config[self.class_names[self.selected_index]])
        else:
            super().keyPressEvent(event)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Package for inventory manipulation
"""
pass



# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for spell factory
"""
from functools import partial

from pyherc.aspects import log_debug, log_info
from pyherc.data.effects import EffectHandle
from pyherc.data.geometry import get_target_in_direction, TargetData
from pyherc.data.magic import Spell
from pyherc.rules.los import get_fov_matrix


class SpellGenerator():
    """
    Factory for creating spells

    .. versionadded:: 0.9
    """
    @log_debug
    def __init__(self):
        """
        Default constructor
        """
        super().__init__()
        self.spell_list = {}

        self.__init_spells()

    @log_debug
    def __init_spells(self):
        """
        Temporary implementation for spell loading
        """

        healing_spell = SpellSpecification([
            EffectHandle(trigger='on spell hit',
                         effect='heal medium wounds',
                         parameters=None,
                         charges=1)],
            targeting_caster,
            spirit=5)

        magic_missile = SpellSpecification([
            EffectHandle(trigger='on spell hit',
                         effect='cause wound',
                         parameters=None,
                         charges=1)],
            targeting_single_target,
            spirit=7)

        fireball = SpellSpecification([
            EffectHandle(trigger='on spell hit',
                         effect='fire',
                         parameters=None,
                         charges=1),
            EffectHandle(trigger='on spell hit',
                         effect='cause wound',
                         parameters=None,
                         charges=1)],
            partial(targeting_spherical_area, radius=3),
            spirit=10)

        self.spell_list['healing wind'] = healing_spell
        self.spell_list['magic missile'] = magic_missile
        self.spell_list['fireball'] = fireball

    @log_info
    def create_spell(self, spell_name, targets):
        """
        Create a spell

        :param spell_name: name of the spell
        :type spell_name: string
        :param target: target of the spell
        :type target: Character
        :returns: ready to use spell
        :rtype: Spell
        """
        new_spell = Spell()
        new_spell.targets.extend(targets)

        spec = self.spell_list[spell_name]
        new_spell.spirit = spec.spirit
        handles = spec.effect_handles

        for effect_handle in handles:
            new_spell.add_effect_handle(effect_handle)

        return new_spell


class SpellSpecification():
    """
    Class to specify spell configuration

    .. versionadded:: 0.9
    """

    @log_debug
    def __init__(self, effect_handles, targeter, spirit):
        super().__init__()
        self.effect_handles = effect_handles
        self.targeter = targeter
        self.spirit = spirit


def targeting_caster(parameters):
    """
    Function to target the caster

    .. versionadded:: 0.9
    """
    return [TargetData('character',
                       parameters.caster.location,
                       parameters.caster,
                       None)]


@log_info
def targeting_single_target(parameters):
    """
    Function to target a single character in direction

    .. versionadded:: 0.9
    """
    target = get_target_in_direction(level=parameters.caster.level,
                                     location=parameters.caster.location,
                                     direction=parameters.direction)
    if target:
        return [target]
    else:
        return []


@log_info
def targeting_spherical_area(parameters, radius):
    """
    Function to target a spherical area

    .. versionadded:: 0.10
    """
    targets = []
    initial = get_target_in_direction(level=parameters.caster.level,
                                      location=parameters.caster.location,
                                      direction=parameters.direction)

    if initial and initial.previous_target:
        splash_center = initial.previous_target.location
        level = parameters.caster.level

        matrix = get_fov_matrix(splash_center,
                                level,
                                radius)

        x_range = range(splash_center[0] - radius,
                        splash_center[0] + radius + 1)

        y_range = range(splash_center[1] - radius,
                        splash_center[1] + radius + 1)

        for x in x_range:
            for y in y_range:
                if matrix[x][y]:
                    creature = level.get_creature_at((x, y))
                    if creature:
                        targets.append(TargetData('character',
                                                  (x, y),
                                                  creature,
                                                  None))
                    elif level.blocks_los(x, y):
                        targets.append(TargetData('wall',
                                                  (x, y),
                                                  None,
                                                  None))
                    else:
                        targets.append(TargetData('void',
                                                  (x, y),
                                                  None,
                                                  None))

    return targets



# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for basic decorators
"""


class Decorator():
    """
    Super class for level decorators
    """
    def __init__(self, configuration):
        """
        Default constructor

        :param configuration: configuration for decorator
        """
        super().__init__()
        self.configuration = configuration

    def __get_level_types(self):
        """
        Get types of levels this decorator supports

        :returns: level types
        :rtype: [string]
        """
        return self.configuration.level_types

    level_types = property(__get_level_types)

    def decorate_level(self, level):
        """
        Decorate level

        :param level: Level to decorate
        :type level: Level
        """
        pass


class DecoratorConfig():
    """
    Super class for decorator configuration
    """
    def __init__(self, level_types):
        """
        Default constructor

        :param level_types: level types to handle
        :type level_types: [string]
        """
        super().__init__()
        self.level_types = level_types


class ReplacingDecorator(Decorator):
    """
    Simple decorator used to replace prototiles with real ones
    """
    def __init__(self, configuration):
        """
        Default constructor

        :param configuration: configuration specifying tiles to replace
        :type configuration: ReplacingDecoratorConfig
        """
        super().__init__(configuration)

    def decorate_level(self, level):
        """
        Decorate level

        :param level: level to decorate
        :type level: Level
        """
        ground_tiles = self.configuration.ground_config
        wall_tiles = self.configuration.wall_config

        for loc_y in range(len(level.floor[0])):
            for loc_x in range(len(level.floor)):
                proto_tile = level.floor[loc_x][loc_y]
                if proto_tile in ground_tiles:
                    level.set_floor_tile(loc_x, loc_y,
                                         ground_tiles[proto_tile])

                proto_tile = level.get_wall_tile(loc_x, loc_y)
                if proto_tile in wall_tiles:
                    level.set_wall_tile(loc_x, loc_y, wall_tiles[proto_tile])


class ReplacingDecoratorConfig(DecoratorConfig):
    """
    Configuration for ReplacingDecorator
    """
    def __init__(self, level_types, ground_config, wall_config):
        """
        Default constructor

        :param level_types: types of level to handle
        :type level_types: [string]
        :param ground_config: configuration for ground
        :param wall_config: configuration for walls
        """
        super().__init__(level_types)
        self.ground_config = ground_config
        self.wall_config = wall_config


class WallBuilderDecorator(Decorator):
    """
    Decorator used to build walls

    This decorator will search all positions where there are walls next to
    empty space and replace tiles there with specified ones
    """
    def __init__(self, configuration):
        """
        Default constructor

        :param configuration: configuration
        :type configuration: WallBuilderDecoratorConfig
        """
        super().__init__(configuration)

    def decorate_level(self, level):
        """
        Decorate level

        :param level: level to decorate
        :type level: Level
        """
        empty_tile = self.configuration.empty_tile

        for loc_y in range(len(level.floor[0])):
            for loc_x in range(len(level.floor)):
                if level.get_wall_tile(loc_x, loc_y) == empty_tile:
                    self.check_and_replace((loc_x - 1, loc_y), level)
                    self.check_and_replace((loc_x + 1, loc_y), level)
                    self.check_and_replace((loc_x, loc_y - 1), level)
                    self.check_and_replace((loc_x, loc_y + 1), level)
                    self.check_and_replace((loc_x - 1, loc_y - 1), level)
                    self.check_and_replace((loc_x - 1, loc_y + 1), level)
                    self.check_and_replace((loc_x + 1, loc_y - 1), level)
                    self.check_and_replace((loc_x + 1, loc_y + 1), level)

    def check_and_replace(self, location, level):
        """
        Check location and replace tile if it matches configuration

        :param location: location to check
        :type location: (integer, integer)
        :param level: level to use
        :type level: Level
        """
        loc_x = location[0]
        loc_y = location[1]
        proto_tile = level.get_wall_tile(loc_x, loc_y)

        if proto_tile in self.configuration.wall_config:
            tile = self.configuration.wall_config[proto_tile]
            level.set_wall_tile(loc_x, loc_y, tile)


class WallBuilderDecoratorConfig(DecoratorConfig):
    """
    Configuration for WallBuilderDecorator
    """
    def __init__(self, level_types, wall_config, empty_tile):
        """
        Default constructor

        :param level_types: types of level to handle
        :type level_types: [string]
        :param wall_config: configuration for walls
        :param empty_tile: tile ID to use for empty spaces
        :type empty_tile: integer
        """
        super().__init__(level_types)
        self.wall_config = wall_config
        self.empty_tile = empty_tile


class AggregateDecorator(Decorator):
    """
    Decorator that consists of multiple decorators
    """
    def __init__(self, configuration):
        """
        Default constructor

        :param configuration: configuration
        :type configuration: AggregateDecoratorConfig
        """
        super().__init__(configuration)

    def decorate_level(self, level):
        """
        Decorate level

        :param level: level to decorate
        :type level: Level
        """
        for decorator in self.configuration.decorators:
            decorator.decorate_level(level)


class AggregateDecoratorConfig(DecoratorConfig):
    """
    Configuration for AggregateDecorator
    """
    def __init__(self, level_types, decorators):
        """
        Default constructor

        :param level_types: types of levels to handle
        :type level_types: [string]
        :decorators: decorators to group
        :type decorators: [Decorator]
        """
        super().__init__(level_types)
        self.decorators = decorators


class DirectionalWallDecoratorConfig(DecoratorConfig):
    """
    Configuration for DirectionalWallDecorator

    .. versionadded:: 0.10
    """
    def __init__(self, level_types, east_west, east_north, east_south,
                 west_north, west_south, north_south,
                 east_west_north, east_west_south,
                 east_north_south, west_north_south,
                 four_way, wall):
        """
        Default constructor
        """
        super().__init__(level_types)
        self.east_west = east_west
        self.east_north = east_north
        self.east_south = east_south
        self.west_north = west_north
        self.west_south = west_south
        self.north_south = north_south

        self.east_west_north = east_west_north
        self.east_west_south = east_west_south
        self.east_north_south = east_north_south
        self.west_north_south = west_north_south
        self.four_way = four_way

        self.wall = wall

        self.tiles = [east_west, east_north, east_south,
                      west_north, west_south, north_south,
                      east_west_north, east_west_south,
                      east_north_south, west_north_south,
                      four_way, wall]


class DirectionalWallDecorator(Decorator):
    """
    Decorator to build directional walls

    .. versionadded:: 0.10
    """
    def __init__(self, configuration):
        """
        Default constructor

        :param configuration: configuration
        :type configuration: DirectionalWallDecoratorConfig
        """
        super().__init__(configuration)

        self.tiles = {'1': configuration.north_south,
                      '13': configuration.east_north,
                      '15': configuration.north_south,
                      '17': configuration.west_north,
                      '135': configuration.east_north_south,
                      '137': configuration.east_west_north,
                      '157': configuration.west_north_south,
                      '1357': configuration.four_way,
                      '3': configuration.east_west,
                      '35': configuration.east_south,
                      '37': configuration.east_west,
                      '357': configuration.east_west_south,
                      '5': configuration.north_south,
                      '57': configuration.west_south,
                      '7': configuration.east_west}

    def decorate_level(self, level):
        """
        Decorate level

        :param level: level to decorate
        :type level: Level
        """
        wall = self.configuration.wall
        for loc_y in range(len(level.floor[0])):
            for loc_x in range(len(level.floor)):
                if level.get_wall_tile(loc_x, loc_y) == wall:
                    level.set_wall_tile(loc_x,
                                        loc_y,
                                        self.get_wall_tile(level,
                                                           loc_x,
                                                           loc_y))

    def get_wall_tile(self, level, loc_x, loc_y):
        """
        Calculate correct wall tile

        :param level: level to decorate
        :type level: Level
        :param loc_x: x-coordinate
        :type loc_x: int
        :param loc_y: y-coordinate
        :type loc_y: int
        :returns: new wall tile
        :rtype: int
        """

        directions = []
        if level.get_wall_tile(loc_x, loc_y - 1) in self.configuration.tiles:
            directions.append('1')
        if level.get_wall_tile(loc_x + 1, loc_y) in self.configuration.tiles:
            directions.append('3')
        if level.get_wall_tile(loc_x, loc_y + 1) in self.configuration.tiles:
            directions.append('5')
        if level.get_wall_tile(loc_x - 1, loc_y) in self.configuration.tiles:
            directions.append('7')

        key = ''.join(directions)

        if key in self.tiles:
            return self.tiles[''.join(directions)]
        else:
            return self.configuration.east_west


class FloorBuilderDecoratorConfig(DecoratorConfig):
    """
    Configuration for FloorBuilderDecorator

    .. versionadded:: 0.10
    """
    def __init__(self, level_types,
                 single, north, east, south, west, north_east, north_south,
                 north_west, east_south, east_west, south_west,
                 north_east_south, north_east_west, north_south_west,
                 east_south_west, fourway, floor):
        """
        Default constructor
        """
        super().__init__(level_types)

        self.single = single
        self.north = north
        self.east = east
        self.south = south
        self.west = west
        self.north_east = north_east
        self.north_south = north_south
        self.north_west = north_west
        self.east_south = east_south
        self.east_west = east_west
        self.south_west = south_west
        self.north_east_south = north_east_south
        self.north_east_west = north_east_west
        self.east_south_west = east_south_west
        self.north_south_west = north_south_west
        self.fourway = fourway
        self.floor = floor

        self.tiles = [single, north, east, south, west, north_east,
                      north_south, north_west, east_south, east_west,
                      south_west, north_east_south, north_east_west,
                      north_south_west, east_south_west, fourway, floor]


class FloorBuilderDecorator(Decorator):
    """
    Decorator to build floors

    .. versionadded:: 0.10
    """
    def __init__(self, configuration):
        """
        Default constructor

        :param configuration: configuration
        :type configuration: FloorBuilderDecoratorConfig
        """
        super().__init__(configuration)

        self.tiles = {'': configuration.single,
                      '1': configuration.north,
                      '3': configuration.east,
                      '5': configuration.south,
                      '7': configuration.west,
                      '13': configuration.north_east,
                      '15': configuration.north_south,
                      '17': configuration.north_west,
                      '35': configuration.east_south,
                      '37': configuration.east_west,
                      '57': configuration.south_west,
                      '135': configuration.north_east_south,
                      '137': configuration.north_east_west,
                      '157': configuration.north_south_west,
                      '357': configuration.east_south_west,
                      '1357': configuration.fourway}

    def decorate_level(self, level):
        """
        Decorate level

        :param level: level to decorate
        :type level: Level
        """
        floor = self.configuration.floor
        for loc_y in range(len(level.floor[0])):
            for loc_x in range(len(level.floor)):
                if level.get_floor_tile(loc_x, loc_y) == floor:
                    level.set_floor_tile(loc_x,
                                         loc_y,
                                         self.get_floor_tile(level,
                                                             loc_x,
                                                             loc_y))

    def get_floor_tile(self, level, loc_x, loc_y):
        """
        Calculate correct floor tile

        :param level: level to decorate
        :type level: Level
        :param loc_x: x-coordinate
        :type loc_x: int
        :param loc_y: y-coordinate
        :type loc_y: int
        :returns: new floor tile
        :rtype: int
        """

        directions = []
        if level.get_floor_tile(loc_x, loc_y - 1) in self.configuration.tiles:
            directions.append('1')
        if level.get_floor_tile(loc_x + 1, loc_y) in self.configuration.tiles:
            directions.append('3')
        if level.get_floor_tile(loc_x, loc_y + 1) in self.configuration.tiles:
            directions.append('5')
        if level.get_floor_tile(loc_x - 1, loc_y) in self.configuration.tiles:
            directions.append('7')

        key = ''.join(directions)

        if key in self.tiles:
            return self.tiles[''.join(directions)]
        else:
            return self.configuration.east_west


class WallOrnamentDecorator(Decorator):
    """
    Decorator to place ornaments of walls

    .. versionadded:: 0.10
    """
    def __init__(self, configuration):
        """
        Default constructor

        :param configuration: configuration to use
        :type configuration: WallOrnamentDecoratorConfig
        """
        super().__init__(configuration)

    def decorate_level(self, level):
        """
        Decorate level

        :param level: level to decorate
        :type level: Level
        """
        wall_tile = self.configuration.wall_tile
        rate = self.configuration.rate

        for loc_y in range(len(level.floor[0])):
            for loc_x in range(len(level.floor)):
                if (level.get_wall_tile(loc_x, loc_y) == wall_tile
                        and self.configuration.rng.randint(0, 100) < rate
                        and level.get_floor_tile(loc_x, loc_y + 1)):
                    self._randomize_ornament(level, loc_x, loc_y)

    def _randomize_ornament(self, level, loc_x, loc_y):
        """
        Set random ornament to given location

        :param level: level to ornament
        :type level: Level
        :param loc_x: x-coordinate
        :type loc_x: int
        :param loc_y: y-coordinate
        :type loc_y: int
        """
        rng = self.configuration.rng
        ornaments = self.configuration.ornamentation

        level.ornamentations[loc_x][loc_y] = rng.choice(ornaments)


class WallOrnamentDecoratorConfig(DecoratorConfig):
    """
    Configuration for WallOrnamentDecorator

    .. versionadded:: 0.10
    """
    def __init__(self, level_types, wall_tile, ornamentation, rng, rate):
        """
        Default constructor

        :param level_types: types of levels to decorate
        :type level_types: [string]
        :param wall_tile: wall to decorate
        :type wall_tile: string
        :param ornamentation: ornamentation to place
        :type ornamentation: string
        :param rng: random number generator
        :type rng: Random
        :param rate: rate of ornamentation
        :type rate: integer 0 <= rate <= 100
        """
        super().__init__(level_types)
        self.wall_tile = wall_tile
        self.ornamentation = ornamentation
        self.rng = rng
        self.rate = rate



# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for small widgets
"""
from PyQt4.QtGui import (QDockWidget, QFrame, QGridLayout, QHBoxLayout, QLabel,
                         QVBoxLayout, QWidget)


class HitPointsWidget(QWidget):
    """
    Widget to show hitpoints

    .. versionadded:: 0.7
    """
    def __init__(self, parent):
        """
        Default constructor

        :param parent: parent of this widget
        :type parent: QWidget
        """
        super().__init__(parent)

        self.hp = None
        self.hp_label = None
        self.spirit = None
        self.spirit_label = None
        self.__set_layout()

    def __set_layout(self):
        """
        Set layout of this widget
        """
        layout = QGridLayout()

        self.hp_label = QLabel()
        self.hp_label.setText('hp: ')
        self.hp_label.setObjectName('no_border')
        self.hp = QLabel()
        self.hp.setText('0 / 0')
        self.hp.setObjectName('no_border')

        self.spirit_label = QLabel()
        self.spirit_label.setText('spirit: ')
        self.spirit_label.setObjectName('no_border')
        self.spirit = QLabel()
        self.spirit.setText('0 / 0')
        self.spirit.setObjectName('no_border')

        layout.addWidget(self.hp_label, 0, 0)
        layout.addWidget(self.hp, 0, 1)
        layout.addWidget(self.spirit_label, 1, 0)
        layout.addWidget(self.spirit, 1, 1)

        self.setLayout(layout)

    def receive_update(self, event):
        """
        Receive update from entity
        """
        if event.event_type == 'hit points changed':
            self.show_hit_points(event.character)
        elif event.event_type == 'spirit points changed':
            self.show_spirit_points(event.character)

    def show_hit_points(self, character):
        """
        Show hit points of given character

        :param character: character whose hit points to show
        :type character: Character
        """
        current_hp = character.hit_points
        max_hp = character.max_hp
        self.hp.setText(str(current_hp) + '/' + str(max_hp))

    def show_spirit_points(self, character):
        """
        Show spirit points of given character

        :param character: character whose spirit points to show
        :type character: Character

        .. versionadded:: 0.10
        """
        self.spirit.setText(str(character.spirit) +
                            '/' +
                            str(character.max_spirit))

class DockingHitPointsWidget(QDockWidget):
    """
    Widget to dock and show hitpoints

    .. versionadded:: 0.7
    """
    def __init__(self, parent):
        """
        Default constructor

        :param parent: parent of this widget
        :type parent: QWidget
        """
        super().__init__(parent)
        self.counter = None

        self.__set_layout()

    def __set_layout(self):
        """
        Set layout of this widget
        """
        self.counter = HitPointsWidget(self)
        self.setWidget(self.counter)

    def receive_update(self, event):
        """
        Receive update from entity
        """
        self.counter.receive_update(event)

    def show_hit_points(self, character):
        """
        Show hit points of given character

        :param character: character whose hit points to show
        :type character: Character
        """
        self.counter.show_hit_points(character)

class ListViewItem(QWidget):
    """
    Widget to show icon, title and description in a list control

    .. versionadded:: 0.7
    """
    def __init__(self, icon = None, title = '', description = ''):
        """
        Default constructor

        :param icon: icon to show
        :type icon: QPixmap
        :param title: title to show
        :type title: string
        :param description: description to show
        :type description: string
        """
        super().__init__()

        self._icon = None
        self._title = None
        self._description = None

        self.__set_layout(icon = icon,
                          title = title,
                          description = description)

    def __set_layout(self, icon, title, description):
        """
        Set layout of this widget
        """
        self._icon = QLabel()
        self._icon.setObjectName('no_border')
        self._title = QLabel()
        self._title.setText(title)
        self._title.setWordWrap(True)
        self._title.setObjectName('charactermenu_skill_title')
        self._description = QLabel()
        self._description.setText(description)
        self._description.setWordWrap(True)
        self._description.setObjectName('no_border')
        self._icon.setPixmap(icon)
        self._icon.setMaximumSize(32, 32)

        frame = QFrame()
        frame.setObjectName('effect')

        outer_layout = QVBoxLayout()
        frame_layout = QVBoxLayout()
        main_layout = QVBoxLayout()
        top_layout = QHBoxLayout()
        bottom_layout = QHBoxLayout()

        top_layout.addWidget(self._icon)

        top_layout.addWidget(self._title)
        bottom_layout.addWidget(self._description)

        frame_layout.addLayout(top_layout)
        frame_layout.addLayout(bottom_layout)
        frame.setLayout(frame_layout)

        outer_layout.addWidget(frame)

        main_layout.addLayout(outer_layout)

        self.setLayout(main_layout)

    def __get_title(self):
        return self._title.text()

    def __get_description(self):
        return self._description.text()

    def __get_icon(self):
        return self._icon.pixmap()

    title = property(__get_title)
    description = property(__get_description)
    icon = property(__get_icon)

class ListView(QWidget):
    """
    Widget to show ListViewItems

    .. versionadded:: 0.7
    """
    def __init__(self):
        """
        Default constructor
        """
        super().__init__()
        self._set_layout()

    def _set_layout(self):
        """
        Set layout of this widget
        """
        layout = QVBoxLayout()
        layout.addStretch()
        self.setLayout(layout)

    def add_item(self, title, description, icon):
        """
        Add item to list view

        :param title: title to show
        :type title: string
        :param description: description to show
        :type description: string
        :param icon: icon to show
        :type icon: QPixmap
        """
        new_item = ListViewItem(title = title,
                                description = description,
                                icon = icon)
        self.layout().insertWidget(self.layout().count() - 1,
                                   new_item)

class EffectsWidget(QWidget):
    """
    Widget to show current active effects

    .. versionadded:: 0.7
    """
    def __init__(self, parent, surface_manager):
        """
        Default constructor
        """
        super().__init__(parent)
        self.main_layout = None
        self.surface_manager = surface_manager
        self._set_layout()

    def _set_layout(self):
        """
        Set layout of this widget
        """
        self.main_layout = QHBoxLayout()
        self.setLayout(self.main_layout)

    def receive_update(self, event):
        """
        Receive update from entity
        """
        if event.event_type in ('heal started', 'poisoned',
                                'heal ended', 'poison ended',
                                'damage started', 'damage ended'):
            for i in range(self.layout().count()):
                self.layout().itemAt(i).widget().close()

            character = event.target
            for effect in character.get_effects():
                new_icon = QLabel()
                new_icon.setPixmap(self.surface_manager.get_icon(effect.icon))
                self.layout().insertWidget(self.layout().count() - 1,
                                       new_icon)

class SpellSelectorWidget(QWidget):
    """
    Widget for selecting spell

    .. versionadded:: 0.10
    """
    def __init__(self, parent, surface_manager):
        """
        Default constructor
        """
        super().__init__(parent)
        self.surface_manager = surface_manager
        self._set_layout()

    def _set_layout(self):
        """
        Set layout of this widget
        """
        self.main_layout = QHBoxLayout()
        self.icon = QLabel()
        self.main_layout.addWidget(self.icon)
        self.icon.setPixmap(self.surface_manager.get_icon('transparent'))
        self.setLayout(self.main_layout)

    def next_spell(self):
        """
        Select next spell
        """
        self.icon.setPixmap(self.surface_manager.get_icon('fire effect'))

    def previous_spell(self):
        """
        Select previous spell
        """
        self.icon.setPixmap(self.surface_manager.get_icon('minor poison'))

    def current_spell(self):
        """
        Currently selected spell

        :returns: currently selected spell
        :rtype: String
        """
        return "fireball"

class AnimatedLabel(QLabel):
    """
    Label to display an animated icon

    .. versionadded:: 0.10
    """
    def __init__(self, timer = None):
        """
        Default constructor
        """
        super().__init__()
        self.tiles = []
        if timer:
            timer.register(self)

    def setPixmap(self, icon):
        """
        Set picture shown in this label
        """
        if hasattr(icon, 'alphaChannel'):
            super().setPixmap(icon)
        else:
            super().setPixmap(icon[0])
            self.tiles = icon

    def animate(self, frame):
        """
        Move animation to given frame

        .. versionadded:: 0.10
        """
        self.setPixmap(self.tiles[frame])

class TimerAdapter():
    """
    Class to trigger animations on glyphs

    .. versionadded:: 0.10
    """
    def __init__(self):
        """
        Default constructor
        """
        self.glyphs = []
        self.frame = 0

    def register(self, glyph):
        """
        Register glyph to internal list
        """
        self.glyphs.append(glyph)

    def trigger_animations(self):
        """
        Process timer event
        """
        self.frame = self.frame + 1
        if self.frame > 1:
            self.frame = 0

        for glyph in self.glyphs:
            glyph.animate(self.frame)



# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Public interface for inventory rules package
"""

from pyherc.aspects import log_debug, log_info
from pyherc.rules.public import ActionParameters


@log_info
def pick_up(character, item, action_factory):
    """
    Pick up item

    :param character: character picking up the item
    :type character: Character
    :param item: item to pick up
    :type item: Item
    :param action_factory: factory to create actions
    :type action_factory: ActionFactory

    .. versionadded:: 0.4
    """
    action = action_factory.get_action(InventoryParameters(character,
                                                           item,
                                                           'pick up'))
    action.execute()


@log_info
def drop_item(character, item, action_factory):
    """
    Drop item from inventory

    :param character: character dropping the item
    :type character: Character
    :param item: item to drop
    :type item: Item
    :param action_factory: factory to create actions
    :type action_factory: ActionFactory

    .. versionadded:: 0.5
    """
    action = action_factory.get_action(InventoryParameters(character,
                                                           item,
                                                           'drop'))
    action.execute()


@log_info
def equip(character, item, action_factory):
    """
    Wear item from inventory

    :param character: character equipping the item
    :type character: Character
    :param item: item to equip
    :type item: Item
    :param action_factory: factory to create actions
    :type action_factory: ActionFactory

    .. versionadded:: 0.8
    """
    action = action_factory.get_action(InventoryParameters(character,
                                                           item,
                                                           'equip'))
    action.execute()


@log_info
def unequip(character, item, action_factory):
    """
    Unequip item

    :param character: character unequipping the item
    :type character: Character
    :param item: item to unequip
    :type item: Item
    :param action_factory: factory to create actions
    :type action_factory: ActionFactory

    .. versionadded:: 0.8
    """
    action = action_factory.get_action(InventoryParameters(character,
                                                           item,
                                                           'unequip'))
    action.execute()


class InventoryParameters(ActionParameters):
    """
    Class for controlling inventory action creation
    """
    @log_debug
    def __init__(self, character, item, sub_action):
        """
        Construct inventory parameters

        :param character: character manipulating the item
        :type character: Character
        :param item: item to manipulate
        :type item: Item
        :param sub_action: type of action to perform
        :type sub_action: string
        """
        super().__init__()

        self.action_type = 'inventory'
        self.sub_action = sub_action
        self.character = character
        self.item = item



# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

# flake8: noqa

"""
Module for level decorators
"""

from .basic import ReplacingDecorator, ReplacingDecoratorConfig
from .basic import WallBuilderDecorator, WallBuilderDecoratorConfig
from .basic import AggregateDecorator, AggregateDecoratorConfig
from .basic import DirectionalWallDecorator, DirectionalWallDecoratorConfig
from .basic import FloorBuilderDecorator, FloorBuilderDecoratorConfig
from .basic import WallOrnamentDecorator, WallOrnamentDecoratorConfig



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for displaying end screen
"""
from datetime import date

from PyQt4.QtGui import QDialog, QLabel, QVBoxLayout


class EndScreen(QDialog):
    """
    Dialog to show end screen

    .. versionadded:: 0.8
    """
    def __init__(self, model, config, dying_rules, parent, flags,
                 controller):
        """
        Default constructor
        """
        super(EndScreen, self).__init__(parent, flags)

        self.name_label = None
        self.keymap = None
        self.result_label = None
        self.instruction_label = None
        self.date_label = None
        self.score_label = None
        self.dying_rules = dying_rules
        self.controller = controller

        self.__set_layout(model,
                          config,
                          parent)

    def __set_layout(self, model, config, parent):
        """
        Set layout of this widget
        """
        self.keymap = self._construct_keymap(config)

        layout = QVBoxLayout()
        self.name_label = QLabel('Name:' + model.player.name)
        self.name_label.setObjectName('no_border')
        self.date_label = QLabel('Date: ' + str(date.today()))
        self.date_label.setObjectName('no_border')
        self.score_label = QLabel('Score: ' + str(self.dying_rules.calculate_score(model.player)))
        self.score_label.setObjectName('no_border')
        self.result_label = QLabel(self.controller.get_end_description(model.end_condition))
        self.result_label.setObjectName('no_border')
        self.instruction_label = QLabel('Press action A to continue')
        self.instruction_label.setObjectName('no_border')

        layout.addWidget(self.name_label)
        layout.addWidget(self.date_label)
        layout.addWidget(self.score_label)
        layout.addWidget(self.result_label)
        layout.addWidget(self.instruction_label)

        self.setLayout(layout)

    def _construct_keymap(self, config):
        """
        Construct key map

        .. versionadded:: 0.8
        """
        keymap = {}

        for key in config.action_a:
            keymap[key] = self._close_window

        return keymap

    def _close_window(self):
        """
        Close the window
        """
        self.done(0)

    def keyPressEvent(self, event):
        """
        Handle keyboard events
        """
        key = event.key()

        if key in self.keymap:
            self.keymap[key]()
        else:
            super(EndScreen, self).keyPressEvent(event)



# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module defining spell casting actions
"""
from pyherc.aspects import log_debug, log_info


class SpellCastingAction():
    """
    Action for casting a spell

    .. versionadded:: 0.9
    """
    @log_debug
    def __init__(self, caster, spell, effects_factory, dying_rules):
        """
        Default constructor

        :param caster: character casting the spell
        :type caster: Character
        :param spell: spell being cast
        :type spell: Spell
        :param effects_factory: factory for creating effects
        :type effects_factory: EffectsFactory
        :param dying_rules: rules for dying
        :type dying_rules: DyingRules
        """
        super().__init__()
        self.caster = caster
        self.spell = spell
        self.effects_factory = effects_factory
        self.dying_rules = dying_rules

    @log_info
    def execute(self):
        """
        Executes this action
        """
        self.caster.spirit = self.caster.spirit - self.spell.spirit

        self.spell.cast(effects_factory=self.effects_factory,
                        dying_rules=self.dying_rules)

    @log_debug
    def is_legal(self):
        """
        Check if the action is possible to perform

        :returns: True if casting spell is possible, false otherwise
        :rtype: Boolean
        """
        return self.caster.spirit - self.spell.spirit >= 0


class GainDomainAction():
    """
    Action for gaining a domain

    .. versionadded:: 0.10
    """
    @log_debug
    def __init__(self, character, item, domain):
        """
        Default constructor

        :param character: character gaining a domain
        :type character: Character
        :param item: item to sacrifice
        :type item: Item
        :param domain: domain to gain
        :type domain: string
        """
        super().__init__()
        self.character = character
        self.item = item
        self.domain = domain

    @log_info
    def execute(self):
        """
        Executes this action
        """
        self.character.add_domain_level(domain=self.domain)

    @log_debug
    def is_legal(self):
        """
        Check if the action is possible to perform

        :returns: True if gaining a domain is possible
        :rtype: Boolean
        """
        return True



# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Public interface for action subsystem

Classes:
ActionFactory - Class used to contruct Action objects
ActionParameters - Class used to guide Action construction

AttackParameters - Class used to guide contruction of attack related actions
DrinkParameters - Class used to guide drinking related actions
InventoryParameters - Class used to guide inventory related actions
SpellCastingParameteres - Class used to guide spell casting
"""

from pyherc.aspects import log_debug, log_info


class ActionFactory():
    """
    Object for creating actions
    """

    @log_debug
    def __init__(self, model, factories):
        """
        Construct ActionFactory

        Args:
            model: model to register for the factory
            factories: a single Factory or list of Factories to use
        """
        super().__init__()

        if hasattr(factories, '__iter__'):
            self.factories = factories
        else:
            self.factories = []
            self.factories.append(factories)

        self.model = model

    @log_info
    def get_action(self, parameters):
        """
        Create an action

        Args:
            parameters: Parameters used to control action creation
        """
        factory = self.get_sub_factory(parameters)
        assert factory is not None, 'suitable factory not configured'
        return factory.get_action(parameters)

    @log_debug
    def get_sub_factories(self):
        """
        Get all sub factories

        Returns:
            List of sub factories
        """
        return self.factories

    @log_debug
    def get_sub_factory(self, parameters):
        """
        Get sub factory to handle parameters

        Args:
            parameters: Parameters to use for searching the factory

        Returns:
            Sub factory if found, None otherwise
        """
        subs = [x for x in self.factories if x.can_handle(parameters)]

        if len(subs) == 1:
            return subs[0]
        else:
            return None


class ActionParameters():
    """
    Object for controlling action creation
    """

    @log_debug
    def __init__(self):
        """
        Default constructor
        """
        super().__init__()
        self.action_type = 'default'

    @log_debug
    def __str__(self):
        """
        Get string representation of this object
        """
        return 'action parameters'



# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Classes for generating square rooms
"""

import logging
from random import Random

from pyherc.generators.level.room.corridor import CorridorGenerator


class SquareRoomGenerator():
    """
    Class for generating a square room
    """
    def __init__(self, floor_tile, empty_tile, corridor_tile, level_types):
        """
        Default constructor

        :param floor_tile: id of the tile to use for floors
        :type floor_tile: integer
        :param empty_tile: id of the empty wall tile
        :type empty_tile: integer
        :param corridor_tile: id of corridor floor tile
        :type corridor_tile: integer
        :param level_types: types of level this generator can be used
        :type level_types: [string]
        """
        self.floor_tile = floor_tile
        self.corridor_tile = corridor_tile
        self.empty_tile = empty_tile
        self.room_width = None
        self.room_height = None
        self.level_types = level_types
        self.rng = Random()
        self.room_corners = None
        self.logger = logging.getLogger('pyherc.generators.level.room.squareroom.SquareRoomGenerator')  # noqa

    def generate_room(self, section):
        """
        Generate room

        :param section: section for generator to draw to
        :type section: Section
        """

        middle_height = section.height // 2
        middle_width = section.width // 2

        if len([x for x in section.connections
                if x.direction == 'right']) > 0:
            room_left_edge = self.rng.randint(2, middle_width - 2)
        else:
            room_left_edge = 1

        if len([x for x in section.connections
                if x.direction == 'left']) > 0:
            room_right_edge = self.rng.randint(middle_width + 2,
                                               section.width - 2)
        else:
            room_right_edge = section.width - 1

        if len([x for x in section.connections
                if x.direction == 'down']) > 0:
            room_top_edge = self.rng.randint(2, middle_height - 2)
        else:
            room_top_edge = 1

        if len([x for x in section.connections
                if x.direction == 'up']) > 0:
            room_bottom_edge = self.rng.randint(middle_height + 2,
                                                section.height - 2)
        else:
            room_bottom_edge = section.height - 1

        for loc_y in range(room_top_edge + 1, room_bottom_edge):
            for loc_x in range(room_left_edge + 1, room_right_edge):
                section.set_floor((loc_x, loc_y), self.floor_tile, 'room')
                section.set_wall((loc_x, loc_y), self.empty_tile, None)

        center_x = (room_right_edge - room_left_edge) // 2 + room_left_edge
        center_y = (room_bottom_edge - room_top_edge) // 2 + room_top_edge

        section.add_room_connection((center_x, room_top_edge), "up")
        section.add_room_connection((center_x, room_bottom_edge), "down")
        section.add_room_connection((room_left_edge, center_y), "left")
        section.add_room_connection((room_right_edge, center_y), "right")

        self.add_corridors(section)

        self.room_corners = []
        self.room_corners.append((room_left_edge + 1, room_top_edge + 1))
        self.room_corners.append((room_right_edge - 1, room_top_edge + 1))
        self.room_corners.append((room_right_edge - 1, room_bottom_edge - 1))
        self.room_corners.append((room_left_edge + 1, room_bottom_edge - 1))

    def add_corridors(self, section):
        """
        Add corridors leading from room connection to section connections

        :param section: section to add corridors
        :type section: Section
        """
        for section_connection in section.connections:
            room_connection = self.find_room_connection(section,
                                                        section_connection)
            corridor = CorridorGenerator(
                room_connection,
                section_connection.translate_to_section(),
                self.empty_tile,
                self.corridor_tile)
            corridor.generate()

    def find_room_connection(self, section, section_connection):
        """
        Find room connection that matches to given section connection

        :param section: section to handle
        :type section: Section
        :param section_connection: connection at the edge of section
        :type section_connection: Connection
        :returns: matching room connection
        :rtype: Connection
        """
        return section.find_room_connection(section_connection)



# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Simple AI for flocking creature

Creature will try to find friends, before attacking the player character
"""

import math

from pyherc.aspects import log_debug
from pyherc.data.geometry import find_direction
from pyherc.rules import attack, is_move_legal, move


class FlockingHerbivore():
    """
    AI for flocking herbivore
    Tries to maintain close distance to other animals
    Seeks out player for combat
    """
    @log_debug
    def __init__(self, character):
        """
        Default constructor

        :param character: character to connect
        :type character: Character
        """
        self.character = character

    @log_debug
    def act(self, model, action_factory, rng):
        """
        Trigger this AI to assess the situation and act accordingly

        :param model: model where the character is located
        :type model: Model
        :param action_factory: factory to create actions
        :type action_factory: ActionFactory
        :param rng: random number generator
        :type rng: Random
        """
        shortest_distance = None
        closest_creature = None
        character = self.character
        player = model.player

        del character.short_term_memory[:]

        for creature in character.level.creatures:
            if creature != character:
                loc_x = abs(creature.location[0] - character.location[0])
                loc_y = abs(creature.location[1] - character.location[1])
                distance = math.sqrt(loc_x * loc_x + loc_y * loc_y)
                if shortest_distance != None:
                    if distance < shortest_distance:
                        shortest_distance = distance
                        closest_creature = creature
                else:
                    shortest_distance = distance
                    closest_creature = creature

        if shortest_distance != None:
            if shortest_distance <= 2:
                #seek player instead
                loc_x = abs(player.location[0] - character.location[0])
                loc_y = abs(player.location[1] - character.location[1])
                distance = math.sqrt(loc_x * loc_x + loc_y * loc_y)

                direction = find_direction(character.location,
                                           player.location)

                if distance > 1.42:
                    if is_move_legal(character,
                                     direction,
                                     'walk',
                                     action_factory):
                        move(character, direction, action_factory)
                    else:
                        character.tick = character.tick + 10
                else:
                    #attack
                    attack(character,
                           direction,
                           action_factory,
                           rng)
            else:
                #find direction
                direction = find_direction(character.location,
                                           closest_creature.location)
                if is_move_legal(character,
                                 direction,
                                 'walk',
                                 action_factory):
                    move(character, direction, action_factory)
                else:
                    character.tick = character.tick + 10
        else:
            #we're all alone here
            character.tick = character.tick + 10



# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
module for configuring catacombs
"""
import hy
from herculeum.ai import FlockingHerbivore, SkeletonWarriorAI
from herculeum.ai.firebeetle import FireBeetleAI
from herculeum.ai.rat import RatAI
from pyherc.config.dsl import LevelConfiguration, LevelContext
from pyherc.data.effects import DamageModifier
from pyherc.generators import creature_config, inventory_config
from pyherc.generators.level.creatures import (CreatureAdder,
                                               CreatureAdderConfiguration)
from pyherc.generators.level.decorator import (AggregateDecorator,
                                               AggregateDecoratorConfig,
                                               DirectionalWallDecorator,
                                               DirectionalWallDecoratorConfig,
                                               FloorBuilderDecorator,
                                               FloorBuilderDecoratorConfig,
                                               ReplacingDecorator,
                                               ReplacingDecoratorConfig,
                                               WallBuilderDecorator,
                                               WallBuilderDecoratorConfig,
                                               WallOrnamentDecorator,
                                               WallOrnamentDecoratorConfig)
from pyherc.generators.level.items import ItemAdder, ItemAdderConfiguration
from pyherc.generators.level.partitioners import GridPartitioner
from pyherc.generators.level.portals import PortalAdderConfiguration
from pyherc.generators.level.room import CatacombsGenerator
from pyherc.rules.constants import (CRUSHING_DAMAGE, LIGHT_DAMAGE,
                                    PIERCING_DAMAGE, POISON_DAMAGE)


def init_level(rng, item_generator, creature_generator, level_size, context):
    """
    Initialise upper catacombs

    :returns: level configuration
    :rtype: LevelConfiguration
    """
    surface_manager = context.surface_manager

    floor_natural = 'natural floor'
    floor_rock = surface_manager.add_icon('floor_rock',
                                          ':rock_floor.png',
                                          '.')

    wall_empty = None
    floor_empty = None

    wall_natural = 'natural wall'
    wall_constructed = 'constructed wall'
    wall_ground = surface_manager.add_icon('wall_ground',
                                           ':ground_wall.png',
                                           ' ')
    wall_rock = surface_manager.add_icon('wall_rock',
                                         ':rock_wall.png',
                                         '#')
    stairs_down = surface_manager.add_icon('stairs_down',
                                           ':stairs_down.png',
                                           '>')
    stairs_up = surface_manager.add_icon('stairs_up',
                                         ':stairs_up.png',
                                         '<')

    wall_15 = surface_manager.add_icon('catacombs_wall_15', ':catacombs_wall_2_8.png', '#')
    wall_57 = surface_manager.add_icon('catacombs_wall_57', ':catacombs_wall_2_4.png', '#')
    wall_35 = surface_manager.add_icon('catacombs_wall_35', ':catacombs_wall_2_6.png', '#')
    wall_37 = surface_manager.add_icon('catacombs_wall_37', ':catacombs_wall_4_6.png', '#')
    wall_17 = surface_manager.add_icon('catacombs_wall_17', ':catacombs_wall_8_4.png', '#')
    wall_13 = surface_manager.add_icon('catacombs_wall_13', ':catacombs_wall_8_6.png', '#')

    wall_135 = surface_manager.add_icon('catacombs_wall_135', ':catacombs_wall_2_6_8.png', '#')
    wall_357 = surface_manager.add_icon('catacombs_wall_357', ':catacombs_wall_2_4_6.png', '#')
    wall_1357 = surface_manager.add_icon('catacombs_wall_1357', ':catacombs_wall_2_4_6_8.png', '#')
    wall_137 = surface_manager.add_icon('catacombs_wall_137', ':catacombs_wall_4_6_8.png', '#')
    wall_157 = surface_manager.add_icon('catacombs_wall_157', ':catacombs_wall_2_4_8.png', '#')

    room_generators = [CatacombsGenerator(floor_natural,
                                           wall_empty,
                                           ['upper catacombs',
                                           'lower catacombs'],
                                           rng)]
    level_partitioners = [GridPartitioner(['upper catacombs',
                                           'lower catacombs'],
                                           1,
                                           1,
                                           rng)]

    replacer_config = ReplacingDecoratorConfig(['upper catacombs',
                                                'lower catacombs'],
                                    {floor_natural: floor_rock},
                                    {wall_natural: wall_ground,
                                    wall_constructed: wall_rock})
    replacer = ReplacingDecorator(replacer_config)

    wallbuilder_config = WallBuilderDecoratorConfig(['upper catacombs',
                                                    'lower catacombs'],
                                        {wall_natural: wall_constructed},
                                         wall_empty)
    wallbuilder = WallBuilderDecorator(wallbuilder_config)

    wall_direction_config = DirectionalWallDecoratorConfig(['upper catacombs'],
                                                   east_west = wall_37,
                                                   east_north = wall_13,
                                                   east_south = wall_35,
                                                   west_north = wall_17,
                                                   west_south = wall_57,
                                                   north_south = wall_15,
                                                   east_west_north = wall_137,
                                                   east_west_south = wall_357,
                                                   east_north_south = wall_135,
                                                   west_north_south = wall_157,
                                                   four_way = wall_1357,
                                                   wall = wall_constructed)

    wall_direction_builder = DirectionalWallDecorator(wall_direction_config)

    floor = surface_manager.add_icon('catacombs_floor', ':catacombs_floor.png', ' ')
    floor1 = surface_manager.add_icon('catacombs_floor_1', ':catacombs_floor_1.png', ' ')
    floor3 = surface_manager.add_icon('catacombs_floor_3', ':catacombs_floor_3.png', ' ')
    floor5 = surface_manager.add_icon('catacombs_floor_5', ':catacombs_floor_5.png', ' ')
    floor7 = surface_manager.add_icon('catacombs_floor_7', ':catacombs_floor_7.png', ' ')
    floor13 = surface_manager.add_icon('catacombs_floor_13', ':catacombs_floor_13.png', ' ')
    floor15 = surface_manager.add_icon('catacombs_floor_15', ':catacombs_floor_15.png', ' ')
    floor17 = surface_manager.add_icon('catacombs_floor_17', ':catacombs_floor_17.png', ' ')
    floor35 = surface_manager.add_icon('catacombs_floor_35', ':catacombs_floor_35.png', ' ')
    floor37 = surface_manager.add_icon('catacombs_floor_37', ':catacombs_floor_37.png', ' ')
    floor57 = surface_manager.add_icon('catacombs_floor_57', ':catacombs_floor_57.png', ' ')
    floor135 = surface_manager.add_icon('catacombs_floor_135', ':catacombs_floor_135.png', ' ')
    floor137 = surface_manager.add_icon('catacombs_floor_137', ':catacombs_floor_137.png', ' ')
    floor157 = surface_manager.add_icon('catacombs_floor_157', ':catacombs_floor_157.png', ' ')
    floor357 = surface_manager.add_icon('catacombs_floor_357', ':catacombs_floor_357.png', ' ')
    floor1357 = surface_manager.add_icon('catacombs_floor_1357', ':catacombs_floor_1357.png', ' ')

    floor_config = FloorBuilderDecoratorConfig([],
                                               single = floor,
                                               north = floor1,
                                               east = floor3,
                                               south = floor5,
                                               west = floor7,
                                               north_east = floor13,
                                               north_south = floor15,
                                               north_west = floor17,
                                               east_south = floor35,
                                               east_west = floor37,
                                               south_west = floor57,
                                               north_east_south = floor135,
                                               north_east_west = floor137,
                                               north_south_west = floor157,
                                               east_south_west = floor357,
                                               fourway = floor1357,
                                               floor = floor_natural)
    floor_builder = FloorBuilderDecorator(floor_config)

    torches_tile_f0 = surface_manager.add_icon('catacombs_torches_f0', ':wall_torches_f0.png', '¤')
    torches_tile_f1 = surface_manager.add_icon('catacombs_torches_f1', ':wall_torches_f1.png', '¤')
    torch_tile_f0 = surface_manager.add_icon('catacombs_torch_f0', ':wall_torch_f0.png', '¤')
    torch_tile_f1 = surface_manager.add_icon('catacombs_torch_f1', ':wall_torch_f1.png', '¤')

    torch_ornamenter_config = WallOrnamentDecoratorConfig(
                                                ['upper catacombs',
                                                 'lower catacombs'],
                                                wall_tile = wall_37,
                                                ornamentation = [(torch_tile_f0, torch_tile_f1),
                                                                 (torches_tile_f0, torches_tile_f1)],
                                                rng = rng,
                                                rate = 10)
    torch_ornamenter = WallOrnamentDecorator(torch_ornamenter_config)

    aggregate_decorator_config = AggregateDecoratorConfig(
                                                ['upper catacombs',
                                                'lower catacombs'],
                                                [wallbuilder,
                                                 wall_direction_builder,
                                                 floor_builder,
                                                 torch_ornamenter,
                                                 replacer])

    decorators = [AggregateDecorator(aggregate_decorator_config)]

    item_adder_config = ItemAdderConfiguration(['upper catacombs',
                                               'lower catacombs'])
    item_adder_config.add_item(min_amount = 0,
                               max_amount = 4,
                               type = 'weapon',
                               location = 'room')
    item_adder_config.add_item(min_amount = 0,
                               max_amount = 2,
                               type = 'potion',
                               location = 'room')
    item_adder_config.add_item(min_amount = 1,
                               max_amount = 3,
                               type = 'food',
                               location = 'room')
    item_adder_config.add_item(min_amount = 0,
                               max_amount = 2,
                               type = 'armour',
                               location = 'room')
    item_adder_config.add_item(min_amount = 0,
                               max_amount = 1,
                               type = 'tome',
                               location = 'room')
    item_adders = [ItemAdder(item_generator,
                            item_adder_config,
                            rng)]

    creatures_upper = CreatureAdderConfiguration(['upper catacombs'])
    creatures_upper.add_creature(min_amount = 6,
                                 max_amount = 12,
                                 name = 'rat')
    creatures_upper.add_creature(min_amount = 0,
                                 max_amount = 1,
                                 name = 'skeleton warrior')

    creatures_lower = CreatureAdderConfiguration(['lower catacombs'])
    creatures_lower.add_creature(min_amount = 6,
                                 max_amount = 12,
                                 name = 'rat')
    creatures_lower.add_creature(min_amount = 2,
                                 max_amount = 5,
                                 name = 'fire beetle')
    creatures_lower.add_creature(min_amount = 1,
                                 max_amount = 3,
                                 name = 'skeleton warrior')

    creature_adders = [CreatureAdder(creature_generator,
                                    creatures_upper,
                                    rng),
                       CreatureAdder(creature_generator,
                                    creatures_lower,
                                    rng)
                                    ]

    portal_adder_configurations = [PortalAdderConfiguration(
                                        icons = (stairs_up,
                                                 stairs_down),
                                        level_type = 'lower catacombs',
                                        location_type = 'room',
                                        chance = 100,
                                        new_level = 'upper catacombs',
                                        unique = True),
                                    PortalAdderConfiguration(
                                        icons = (stairs_up,
                                                 stairs_down),
                                        level_type = 'upper catacombs',
                                        location_type = 'room',
                                        chance = 100,
                                        new_level = '',
                                        unique = True,
                                        escape_stairs = True)
                                        ]

    level_context = LevelContext(size = level_size,
                                 floor_type = floor_empty,
                                 wall_type = wall_natural,
                                 empty_floor = floor_empty,
                                 empty_wall = wall_empty,
                                 level_types = ['upper catacombs',
                                                'lower catacombs'])

    config = (LevelConfiguration()
                    .with_rooms(room_generators)
                    .with_partitioners(level_partitioners)
                    .with_decorators(decorators)
                    .with_items(item_adders)
                    .with_creatures(creature_adders)
                    .with_portals(portal_adder_configurations)
                    .with_contexts([level_context])
                    .build())

    return config

def init_creatures(context):
    """
    Initialise creatures''

    :returns: list of configuration items
    :rtype: [CreatureConfiguration]
    """
    surface_manager = context.surface_manager
    config = []

    mouse_f0 = surface_manager.add_icon('rat_f0', ':mouse_f0.png', 'r', ['yellow', 'dim'])
    mouse_f1 = surface_manager.add_icon('rat_f1', ':mouse_f1.png', 'r', ['yellow', 'dim'])
    config.append(creature_config(name = 'rat',
                                  body = 4,
                                  finesse = 12,
                                  mind = 2,
                                  hp = 2,
                                  speed = 2,
                                  icons = (mouse_f0, mouse_f1),
                                  attack = 1,
                                  ai = RatAI))

    firebeetle_f0 = surface_manager.add_icon('fire beetle_f0', ':scarab-beetle_f0.png', 'a', ['red'])
    firebeetle_f1 = surface_manager.add_icon('fire beetle_f1', ':scarab-beetle_f1.png', 'a', ['red'])
    config.append(creature_config(name = 'fire beetle',
                                  body = 10,
                                  finesse = 11,
                                  mind = 0,
                                  hp = 4,
                                  speed = 3,
                                  icons = (firebeetle_f0, firebeetle_f1),
                                  attack = 2,
                                  ai = FireBeetleAI))

    skeleton_inventory = [inventory_config(item_name = 'sword',
                                           min_amount = 0,
                                           max_amount = 1,
                                           probability = 100)]

    skeleton_effects = [DamageModifier(modifier = 2,
                                       damage_type = CRUSHING_DAMAGE,
                                       duration = None,
                                       frequency = None,
                                       tick = None,
                                       icon = 0,
                                       title = '',
                                       description = ''),
                        DamageModifier(modifier = 2,
                                       damage_type = LIGHT_DAMAGE,
                                       duration = None,
                                       frequency = None,
                                       tick = None,
                                       icon = 0,
                                       title = '',
                                       description = ''),
                        DamageModifier(modifier = -2,
                                       damage_type = PIERCING_DAMAGE,
                                       duration = None,
                                       frequency = None,
                                       tick = None,
                                       icon = 0,
                                       title = '',
                                       description = ''),
                        DamageModifier(modifier = -2,
                                       damage_type = POISON_DAMAGE,
                                       duration = None,
                                       frequency = None,
                                       tick = None,
                                       icon = 0,
                                       title = '',
                                       description = '')]

    skeleton_f0 = surface_manager.add_icon('skeleton warrior_f0', ':blade-bite_f0.png', 'Z', ['white', 'bold'])
    skeleton_f1 = surface_manager.add_icon('skeleton warrior_f1', ':blade-bite_f1.png', 'Z', ['white', 'bold'])
    config.append(creature_config(name = 'skeleton warrior',
                                  body = 8,
                                  finesse = 11,
                                  mind = 0,
                                  hp = 8,
                                  speed = 4,
                                  icons = (skeleton_f0, skeleton_f1),
                                  attack = 1,
                                  ai = SkeletonWarriorAI,
                                  inventory = skeleton_inventory,
                                  effects = skeleton_effects))

    return config



# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Public interface for magic rules package
"""

from pyherc.aspects import log_debug, log_info
from pyherc.rules.public import ActionParameters


@log_info
def cast(character, direction, spell_name, action_factory):
    """
    Cast a spell

    :param character: character casting the spell
    :type character: Character
    :param direction: direction to cast the spell
    :type direction: int
    :param spell: spell to cast
    :type spell: string
    :param action_factory: factory to create actions
    :type action_factory: ActionFactory

    .. versionadded:: 0.9
    """
    action = action_factory.get_action(
        SpellCastingParameters(caster=character,
                               direction=direction,
                               spell_name=spell_name))

    if action.is_legal():
        action.execute()


@log_info
def gain_domain(character, action_factory, domain, item):
    """
    Sacrifice an item to gain a domain

    :param action_factory: factory to create actions
    :type action_factory: ActionFactory
    :param domain: name of the domain to gain
    :type domain: String
    :param item: item to sacrifice
    :type item: Item

    .. versionadded:: 0.10
    """
    action = action_factory.get_action(GainDomainParameters(character,
                                                            domain,
                                                            item))

    if action.is_legal():
        action.execute()


class SpellCastingParameters(ActionParameters):
    """
    Class for controlling spell casting

    .. versionadded:: 0.9
    """

    @log_debug
    def __init__(self, caster, direction, spell_name):
        """
        Construct spell casting parameters

        :param caster: character casting the spell
        :type caster: Character
        :param direction: direction to which the spell is cast
        :type direction: int
        :param spell_name: name of the spell
        :type spell_name: string
        """
        super().__init__()

        self.action_type = 'spell casting'
        self.caster = caster
        self.direction = direction
        self.spell_name = spell_name


class GainDomainParameters(ActionParameters):
    """
    Class for controlling gaining a domain

    .. versionadded:: 0.10
    """
    @log_debug
    def __init__(self, character, item, domain):
        """
        Construct gain domain parameter

        :param character: character to gain a domain
        :type character: Character
        :param item: item to sacrifice
        :type item: Item
        :param domain: domain to gain
        :type domain: String
        """
        super().__init__()

        self.action_type = 'gain domain'
        self.character = character
        self.item = item
        self.domain = domain



# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module defining rules engine
"""


class RulesEngine():
    """
    Engine for rules of the game

    .. versionadded:: 0.6
    """
    def __init__(self, action_factory, dying_rules):
        """
        Default constructor

        :param action_factory: factory for actions
        :type action_factory: ActionFactory
        :param dying: rules for dying
        :type dying: Dying
        """
        super().__init__()

        self.__action_factory = action_factory
        self.__dying_rules = dying_rules

    def __get_action_factory(self):
        """
        Action factory of the rules engine
        """
        return self.__action_factory

    def __get_dying_rules(self):
        """
        Rules for dying
        """
        return self.__dying_rules

    action_factory = property(__get_action_factory)
    dying_rules = property(__get_dying_rules)



# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Classes for damage counters
"""
from PyQt4.QtCore import pyqtProperty, QObject
from PyQt4.QtGui import QColor, QFont, QGraphicsSimpleTextItem


class DamageCounter(QGraphicsSimpleTextItem):
    """
    Counter for showing damage

    .. versionadded:: 0.6
    """
    def __init__(self, damage, colour, parent):
        """
        Default constructor
        """
        super().__init__()

        font = QFont('Helvetica',
                     12,
                     QFont.Bold,
                     False)

        self.setText(str(damage))
        self.setBrush(QColor(colour))
        self.setFont(font)

        self.adapter = DamageCounterAdapter(self, self)


class DamageCounterAdapter(QObject):
    """
    Adapter for damage counter

    .. versionadded:: 0.6
    """
    def __init__(self, parent, object_to_animate):
        """
        Default constructor
        """
        super().__init__()
        self.object_to_animate = object_to_animate

    def __get_y_location(self):
        return self.object_to_animate.y()

    def __set_y_location(self, y):
        self.object_to_animate.setY(y)

    def __get_opacity(self):
        return self.object_to_animate.opacity()

    def __set_opacity(self, opacity):
        self.object_to_animate.setOpacity(opacity)

    y_location = pyqtProperty(int, __get_y_location, __set_y_location)
    opacity = pyqtProperty(float, __get_opacity, __set_opacity)


class MapGlyphAdapter(QObject):
    """
    Adapter for damage counter

    .. versionadded:: 0.12
    """
    def __init__(self, parent, object_to_animate, keep_centered = False):
        """
        Default constructor
        """
        super().__init__()
        self.view = parent.view
        self.object_to_animate = object_to_animate
        self.keep_centered = keep_centered

    def __get_y_location(self):
        return self.object_to_animate.y()

    def __set_y_location(self, y):
        self.object_to_animate.setY(y)

    def __get_x_location(self):
        return self.object_to_animate.x()

    def __set_x_location(self, x):
        self.object_to_animate.setX(x)
        if self.keep_centered:
            self._center()

    def _center(self):
        if self.object_to_animate.flipped:
            off_set = 0
        else:
            off_set = 32

        self.view.setSceneRect(self.object_to_animate.x() - 180 + off_set,
                               self.object_to_animate.y() - 180,
                               320,
                               320)

    y_location = pyqtProperty(int, __get_y_location, __set_y_location)
    x_location = pyqtProperty(int, __get_x_location, __set_x_location)



# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Classes for magic damage
"""
from .animation import Animation
from .counters import MapGlyphAdapter
from PyQt4.QtCore import QPropertyAnimation, QParallelAnimationGroup
from PyQt4.QtCore import QAbstractAnimation
from PyQt4.QtGui import QTransform


class MoveAnimation(Animation):
    """
    Animation for moving

    .. versionadded:: 0.12
    """
    def __init__(self, event):
        """
        Default constructor
        """
        super().__init__(event)

        self.start = event.old_location
        self.destination = event.mover.location
        self.direction = event.direction
        self.mover = event.mover

        self.flipped = False
        self.offset = 0

    def trigger(self, ui):
        """
        Trigger this animation
        """
        glyph = [glyph for glyph in ui.scene.items()
                 if hasattr(glyph, 'entity') and glyph.entity == self.mover][0]

        if self.direction in (2, 3, 4):
            self.flipped = True
            self.offset = 32

        if self.direction in (6, 7, 8):
            self.flipped = False
            self.offset = 0

        if self.flipped:
            glyph.flipped = True
            glyph.setTransform(QTransform.fromScale(-1, 1))
        else:
            glyph.flipped = False
            glyph.setTransform(QTransform.fromScale(1, 1))

        if glyph.entity.artificial_intelligence:
            adapter = MapGlyphAdapter(ui, glyph)
        else:
            adapter = MapGlyphAdapter(ui, glyph, True)

        animation = QParallelAnimationGroup()

        move_y = QPropertyAnimation(adapter, 'y_location')
        move_y.setDuration(100)
        move_y.setStartValue(self.start[1] * 32)
        move_y.setEndValue(self.destination[1] * 32)

        move_x = QPropertyAnimation(adapter, 'x_location')
        move_x.setDuration(100)
        move_x.setStartValue(self.start[0] * 32 + self.offset)
        move_x.setEndValue(self.destination[0] * 32 + self.offset)

        animation.addAnimation(move_y)
        animation.addAnimation(move_x)

        ui.animations.append(animation)
        animation.finished.connect(ui.remove_finished_animation)

        animation.start()



# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Spell casting related factories
"""
from pyherc.aspects import log_debug, log_info
from pyherc.rules.factory import SubActionFactory
from pyherc.rules.magic.action import GainDomainAction, SpellCastingAction


class SpellCastingFactory(SubActionFactory):
    """
    Factory for creating spell casting actions

    .. versionadded:: 0.9
    """
    @log_debug
    def __init__(self, spell_factory, effects_factory, dying_rules):
        """
        Constructor for this factory
        """
        super().__init__()
        self.spell_factory = spell_factory
        self.effects_factory = effects_factory
        self.dying_rules = dying_rules
        self.action_type = 'spell casting'

    @log_info
    def get_action(self, parameters):
        """
        Create a spell casting action

        :param parameters: parameters used to control creation
        :type parameters: SpellCastingParameters
        """
        spell = self.spell_factory.create_spell(
            spell_name=parameters.spell_name,
            targets=self._get_spell_targets(parameters))

        return SpellCastingAction(caster=parameters.caster,
                                  spell=spell,
                                  effects_factory=self.effects_factory,
                                  dying_rules=self.dying_rules)

    @log_debug
    def _get_spell_targets(self, parameters):
        """
        Get targets for spell

        :returns: targets of a spell
        :rtype: [Character]
        """
        spec = self.spell_factory.spell_list[parameters.spell_name]
        return spec.targeter(parameters)


class GainDomainFactory(SubActionFactory):
    """
    Factory for creating gain domain actions

    .. versionadded:: 0.10
    """
    @log_debug
    def __init__(self):
        """
        Constructor for this factory
        """
        super().__init__()
        self.action_type = 'gain domain'

    @log_info
    def get_action(self, parameters):
        """
        Create a gain domain action

        :param parameters: parameters used to control creation
        :type parameters: GainDomainParameters
        """
        return GainDomainAction(character=parameters.character,
                                item=parameters.item,
                                domain=parameters.domain)



# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for displaying inventory
"""
from herculeum.ui.controllers import InventoryController
from pyherc.data import Item
from PyQt4.QtCore import pyqtSignal, Qt
from PyQt4.QtGui import (QApplication, QFont, QGridLayout, QHBoxLayout, QLabel,
                         QPainter, QPixmap, QTextEdit, QVBoxLayout, QWidget)
from PyQt4.QtSvg import QSvgWidget


class CharacterInventoryWidget(QWidget):
    """
    Widget to show inventory of a character

    .. versionadded:: 0.6
    """
    def __init__(self, surface_manager, character, config, parent):
        """
        Default constructor
        """
        super(CharacterInventoryWidget, self).__init__(parent)

        self.surface_manager = surface_manager
        self.character = character
        self.config = config
        self.items = []
        self.head_slot = None
        self.boots_slot = None
        self.shield_slot = None
        self.weapon_slot = None
        self.character_icon = None
        self.gloves_slot = None
        self.ring_slot = None
        self.arrows_slot = None
        self.necklace_slot = None
        self.belt_slot = None
        self.armour_slot = None


        self.__set_layout(surface_manager, character, parent)
        self.keymap, self.move_keys = self._construct_keymaps(config)

    ItemFocused = pyqtSignal(Item, name='ItemFocused')
    ItemActionA = pyqtSignal(Item, name='ItemActionA')
    ItemActionB = pyqtSignal(Item, name='ItemActionB')
    ItemActionX = pyqtSignal(Item, name='ItemActionX')
    ItemActionY = pyqtSignal(Item, name='ItemActionY')

    def _construct_keymaps(self, config):
        """
        Construct keymaps from configuration

        :param config: controls configuration
        :type config: ControlsConfiguration
        :returns: keymap for widget and move keys
        :rtype: {}, {}

        .. versionadded:: 0.8
        """
        move_keys = {}
        keymap = {}

        for key in config.move_down_left:
            move_keys[key] = 1
            keymap[key] = self._move
        for key in config.move_down:
            move_keys[key] = 1
            keymap[key] = self._move
        for key in config.move_down_right:
            move_keys[key] = 1
            keymap[key] = self._move
        for key in config.move_left:
            move_keys[key] = -1
            keymap[key] = self._move
        for key in config.move_right:
            move_keys[key] = 1
            keymap[key] = self._move
        for key in config.move_up_left:
            move_keys[key] = -1
            keymap[key] = self._move
        for key in config.move_up:
            move_keys[key] = -1
            keymap[key] = self._move
        for key in config.move_up_right:
            move_keys[key] = -1
            keymap[key] = self._move

        for key in config.action_a:
            keymap[key] = self._action
        for key in config.action_b:
            keymap[key] = self._action

        return keymap, move_keys

    def __set_layout(self, surface_manager, character, parent):
        """
        Set layout of this widget
        """
        main_layout = QHBoxLayout()

        left_side = QVBoxLayout()
        self.ring_slot = ItemGlyph(None,
                                   surface_manager,
                                   self,
                                   QPixmap(':ring.png'))
        self.items.append(self.ring_slot)

        self.weapon_slot = ItemGlyph(None,
                                     surface_manager,
                                     self,
                                     QPixmap(':inventory_sword.png'))
        self.items.append(self.weapon_slot)

        self.gloves_slot = ItemGlyph(None,
                                     surface_manager,
                                     self,
                                     QPixmap(':mailed-fist.png'))
        self.items.append(self.gloves_slot)

        left_side.addStretch()
        left_side.addWidget(self.ring_slot)
        left_side.addWidget(self.weapon_slot)
        left_side.addWidget(self.gloves_slot)
        left_side.addStretch()

        middle = QVBoxLayout()

        middle_top = QHBoxLayout()
        self.head_slot = ItemGlyph(None,
                                   surface_manager,
                                   self,
                                   QPixmap(':helm.png'))
        self.items.append(self.head_slot)

        self.necklace_slot = ItemGlyph(None,
                                       surface_manager,
                                       self,
                                       QPixmap(':necklace.png'))
        self.items.append(self.necklace_slot)

        middle_top.addStretch()
        middle_top.addWidget(self.head_slot)
        middle_top.addWidget(self.necklace_slot)
        middle_top.addStretch()

        middle_middle = QHBoxLayout()

        self.character_icon = QSvgWidget(':strong.svg')
        self.character_icon.setMaximumSize(150, 150)
        self.character_icon.setMinimumSize(150, 150)
        middle_middle.addWidget(self.character_icon)

        middle_bottom = QHBoxLayout()
        self.boots_slot = ItemGlyph(None,
                                    surface_manager,
                                    self,
                                    QPixmap(':boots.png'))
        self.items.append(self.boots_slot)

        self.belt_slot = ItemGlyph(None,
                                   surface_manager,
                                   self,
                                   QPixmap(':belts.png'))
        self.items.append(self.belt_slot)

        middle_bottom.addStretch()
        middle_bottom.addWidget(self.boots_slot)
        middle_bottom.addWidget(self.belt_slot)
        middle_bottom.addStretch()

        middle.addLayout(middle_top)
        middle.addLayout(middle_middle)
        middle.addLayout(middle_bottom)

        right_side = QVBoxLayout()
        self.arrows_slot = ItemGlyph(None,
                                     surface_manager,
                                     self,
                                     QPixmap(':arrow-cluster.png'))
        self.items.append(self.arrows_slot)

        self.shield_slot = ItemGlyph(None,
                                     surface_manager,
                                     self,
                                     QPixmap(':shield.png'))
        self.items.append(self.shield_slot)

        self.armour_slot = ItemGlyph(None,
                                     surface_manager,
                                     self,
                                     QPixmap(':breastplate.png'))
        self.items.append(self.armour_slot)

        right_side.addStretch()
        right_side.addWidget(self.arrows_slot)
        right_side.addWidget(self.shield_slot)
        right_side.addWidget(self.armour_slot)
        right_side.addStretch()

        main_layout.addLayout(left_side)
        main_layout.addLayout(middle)
        main_layout.addLayout(right_side)

        self.setLayout(main_layout)

    def show_character(self):
        """
        Show character
        """
        self.weapon_slot.set_item(self.character.inventory.weapon)
        self.armour_slot.set_item(self.character.inventory.armour)
        self.arrows_slot.set_item(self.character.inventory.projectiles)
        self.head_slot.set_item(self.character.inventory.helm)
        self.boots_slot.set_item(self.character.inventory.boots)
        self.shield_slot.set_item(self.character.inventory.shield)
        self.gloves_slot.set_item(self.character.inventory.gloves)
        self.ring_slot.set_item(self.character.inventory.ring)
        self.necklace_slot.set_item(self.character.inventory.necklace)
        self.belt_slot.set_item(self.character.inventory.belt)

    def _move(self, key):
        """
        Handle move keys

        .. versionadded:: 0.8
        """
        current = [x for x in self.items
                   if x.display.objectName() == 'active_inventorybox'][0]

        index = self.items.index(current)
        new_index = index + self.move_keys[key]

        if new_index < len(self.items) and new_index >= 0:
            new = self.items[new_index]
            new.setFocus(Qt.OtherFocusReason)
        else:
            if new_index < 0:
                self.focusPreviousChild()
            else:
                self.focusNextChild()

    def _action(self, key):
        """
        Handle action keys

        .. versionadded:: 0.8
        """
        item = [x for x in self.items
                if x.display.objectName() == 'active_inventorybox'][0].item
        if item != None:
            if key in self.config.action_a:
                self.ItemActionA.emit(item)
            elif key in self.config.action_b:
                self.ItemActionB.emit(item)

    def keyPressEvent(self, event): #pylint: disable-msg=C0103
        """
        Handle keyboard events
        """
        key = event.key()

        if key in self.keymap:
            self.keymap[key](key)
        else:
            super(CharacterInventoryWidget, self).keyPressEvent(event)

    def focus_on_first_item(self):
        """
        Focus on the first item
        """
        self.items[0].setFocus()

    def focus_on_last_item(self):
        """
        Focus on the last item
        """
        self.items[-1].setFocus()

class ItemDescriptionWidget(QWidget):
    """
    Widget to show item description

    .. versionadded:: 0.6
    """
    def __init__(self, parent):
        """
        Default constructor
        """
        super(ItemDescriptionWidget, self).__init__(parent)
        self.setFocusPolicy(Qt.NoFocus)

        self.text_edit = None

        self.__set_layout()

    def __set_layout(self):
        """
        Sets layout of this widget
        """
        layout = QHBoxLayout()
        self.text_edit = QTextEdit()
        self.text_edit.setReadOnly(True)

        layout.addWidget(self.text_edit)
        self.setLayout(layout)

    def set_text(self, text):
        """
        Set text being displayed on this widget
        """
        self.text_edit.setText(text)

class InventoryWidget(QWidget):
    """
    Widget for showing inventory

    .. versionadded:: 0.5
    """
    def __init__(self, surface_manager, character, action_factory,
                 config, parent):
        """
        Default constructor
        """
        super(InventoryWidget, self).__init__(parent)

        self.surface_manager = surface_manager
        self.character = character
        self.action_factory = action_factory
        self.config = config
        self.items_carried = None
        self.items_in_ground = None
        self.item_description = None
        self.character_inventory = None

        self.inventory_controller = InventoryController(character,
                                                        action_factory)

        self.__set_layout(surface_manager, character, config)

    ItemPickedUp = pyqtSignal(Item, name='ItemPickedUp')
    ItemDropped = pyqtSignal(Item, name='ItemDropped')

    def __set_layout(self, surface_manager, character, config):
        """
        Set layout of this widget
        """
        main_layout = QHBoxLayout()

        left_side = QVBoxLayout()
        self.character_inventory = CharacterInventoryWidget(surface_manager,
                                                            character,
                                                            config,
                                                            self)
        self.character_inventory.ItemActionA.connect(self.unwield_weapon)
        self.character_inventory.ItemActionB.connect(self.unwield_weapon)

        self.item_description = ItemDescriptionWidget(self)
        left_side.addWidget(self.character_inventory)
        left_side.addWidget(self.item_description)

        right_side = QVBoxLayout()
        self.items_carried = ItemBox(surface_manager = surface_manager,
                                     config = config,
                                     parent = self,
                                     width = 6,
                                     height = 6)
        self.items_carried.ItemFocused.connect(self.on_item_focused)
        self.items_carried.ItemActionA.connect(self.use_item)
        self.items_carried.ItemActionB.connect(self.drop_item)

        self.items_in_ground = ItemBox(surface_manager = surface_manager,
                                       config = config,
                                       parent = self,
                                       width = 6,
                                       height = 2)
        self.items_in_ground.ItemFocused.connect(self.on_item_focused)
        self.items_in_ground.ItemActionA.connect(self.pick_up_item)

        right_side.addWidget(self.items_carried)
        right_side.addWidget(self.items_in_ground)

        main_layout.addLayout(left_side)
        main_layout.addLayout(right_side)

        self.setLayout(main_layout)

        self.update_inventory()
        self.items_carried.items[0].setFocus(Qt.OtherFocusReason)

    def update_inventory(self):
        """
        Update items being displayed

        .. versionadded:: 0.6
        """
        self.items_carried.show_items(self.character.inventory)
        items = self.character.level.get_items_at(self.character.location)
        self.items_in_ground.show_items(items)
        self.character_inventory.show_character()

    def on_item_focused(self, item):
        """
        Handle item focused

        .. versionadded:: 0.6
        """
        description = self.inventory_controller.item_description(item)
        self.item_description.set_text(description)

    def focusNextPrevChild(self, next): #pylint: disable-msg=C0103
        """
        Handle moving focus around the widget

        .. versionadded:: 0.7
        """
        if len([x for x in self.items_carried.items
                if x.display.objectName() == 'active_inventorybox']) > 0:
            focused = self.items_carried
        elif len([x for x in self.items_in_ground.items
                  if x.display.objectName() == 'active_inventorybox']) > 0:
            focused = self.items_in_ground.items
        else:
            focused = self.character_inventory

        if focused == self.character_inventory:
            if next == True:
                self.items_carried.focus_on_first_row(0)
            else:
                self.items_in_ground.focus_on_bottom_row(0)
        elif focused == self.items_carried:
            if next == True:
                index = self.items_carried.get_current_column()
                self.items_in_ground.focus_on_first_row(index)
            else:
                self.character_inventory.focus_on_last_item()
        else:
            if next == True:
                self.character_inventory.focus_on_first_item()
            else:
                index = self.items_in_ground.get_current_column()
                self.items_carried.focus_on_bottom_row(index)

        return True

    def pick_up_item(self, item):
        """
        Pick up item

        .. versionadded:: 0.6
        """
        self.inventory_controller.pick_up_item(item)
        self.update_inventory()
        self.ItemPickedUp.emit(item)

    def drop_item(self, item):
        """
        Drop item

        .. versionadded:: 0.6
        """
        self.inventory_controller.drop_item(item)
        self.update_inventory()
        self.ItemDropped.emit(item)

    def use_item(self, item):
        """
        Use item in different ways, depending on the item

        .. versionadded:: 0.6
        """
        self.inventory_controller.use_item(item)
        self.update_inventory()

    def unwield_weapon(self, item):
        """
        Unwield current weapon

        .. versionadded:: 0.6
        """
        self.inventory_controller.unequip_item(item)
        self.update_inventory()

class ItemBox(QWidget):
    """
    Widget for displaying many items

    .. versionadded:: 0.5
    """
    def __init__(self, surface_manager, config, parent, width, height):
        """
        Default constructor
        """
        super(ItemBox, self).__init__(parent)

        self.config = config
        self.surface_manager = surface_manager
        self.item_width = width
        self.item_height = height

        self.rows = None
        self.items = None
        self.grid_layout = None

        self.__set_layout(width, height)

        self.keymap, self.move_keys = self._construct_keymaps(config, width)

    ItemFocused = pyqtSignal(Item, name='ItemFocused')
    ItemActionA = pyqtSignal(Item, name='ItemActionA')
    ItemActionB = pyqtSignal(Item, name='ItemActionB')
    ItemActionX = pyqtSignal(Item, name='ItemActionX')
    ItemActionY = pyqtSignal(Item, name='ItemActionY')

    def _construct_keymaps(self, config, width):
        """
        Construct keymaps from configuration

        :param config: controls configuration
        :type config: ControlsConfiguration
        :returns: keymap for widget and move keys
        :rtype: {}, {}

        .. versionadded:: 0.8
        """
        move_keys = {}
        keymap = {}

        for key in config.move_down_left:
            move_keys[key] = width - 1
            keymap[key] = self._move
        for key in config.move_down:
            move_keys[key] = width
            keymap[key] = self._move
        for key in config.move_down_right:
            move_keys[key] = width + 1
            keymap[key] = self._move
        for key in config.move_left:
            move_keys[key] = - 1
            keymap[key] = self._move
        for key in config.move_right:
            move_keys[key] = 1
            keymap[key] = self._move
        for key in config.move_up_left:
            move_keys[key] = -width - 1
            keymap[key] = self._move
        for key in config.move_up:
            move_keys[key] = -width
            keymap[key] = self._move
        for key in config.move_up_right:
            move_keys[key] = -width + 1
            keymap[key] = self._move

        for key in config.action_a:
            keymap[key] = self._action
        for key in config.action_b:
            keymap[key] = self._action

        return keymap, move_keys

    def __set_layout(self, width, height):
        """
        Set layout of this widget
        """
        self.rows = []

        self.grid_layout = QGridLayout()
        self.grid_layout.setSpacing(0)
        self.items = []

        for y in range(0, height):
            for x in range(0, width):
                new_item = ItemGlyph(None,
                                     self.surface_manager,
                                     self)

                self.grid_layout.addWidget(new_item, y, x)
                self.items.append(new_item)
                new_item.ItemFocused.connect(self.on_item_focused)


        self.setLayout(self.grid_layout)

    def keyPressEvent(self, event): #pylint: disable-msg=C0103
        """
        Handle keyboard events

        .. versionadded:: 0.6
        """
        key = event.key()

        if key in self.keymap:
            self.keymap[key](key)
        else:
            super(ItemBox, self).keyPressEvent(event)

    def _action(self, key):
        """
        Handle action keys

        .. versionadded:: 0.8
        """
        item = self.get_current_slot().item
        if item != None:
            if key in self.config.action_a:
                self.ItemActionA.emit(item)
            elif key in self.config.action_b:
                self.ItemActionB.emit(item)

    def _move(self, key):
        """
        Handle move keys

        .. versionadded:: 0.8
        """
        current = self.get_current_slot()

        index = self.items.index(current)

        new_index = index + self.move_keys[key]

        if new_index < len(self.items) and new_index >= 0:
            new = self.items[new_index]
            new.setFocus(Qt.OtherFocusReason)
        else:
            if new_index >= len(self.items):
                self.focusNextChild()
            else:
                self.focusPreviousChild()

    def show_items(self, items):
        """
        Show given items

        .. versionadded:: 0.6
        """
        item_count = len(items)

        for counter in range(0, item_count):
            self.items[counter].set_item(items[counter])

        for counter in range(item_count, len(self.items)):
            self.items[counter].set_item(None)

    def get_current_slot(self):
        """
        Get currently selected slot
        """
        current = [x for x in self.items
                   if x.display.objectName() == 'active_inventorybox']

        if len(current) > 0:
            return current[0]
        else:
            return None

    def get_current_column(self):
        """
        Get index of current column
        """
        slot = self.get_current_slot()

        if slot == None:
            return None

        index = self.items.index(slot)
        column = index % self.item_width

        return column

    def focus_on_first_row(self, index):
        """
        Focus on slot on first row
        """
        self.items[index].setFocus()

    def focus_on_bottom_row(self, index):
        """
        Focus on slot on last row
        """
        self.items[-self.item_width + index].setFocus()

    def on_item_left_selected(self, item):
        """
        Handle left selecting item

        .. versionadded:: 0.6
        """
        self.ItemActionA.emit(item)

    def on_item_right_selected(self, item):
        """
        Handle right selecting item

        .. versionadded:: 0.6
        """
        self.ItemActionB.emit(item)

    def on_item_focused(self, item):
        """
        Handle item focusing

        .. versionadded:: 0.6
        """
        self.ItemFocused.emit(item)

class ItemGlyph(QWidget):
    """
    Class to display item on screen

    .. versionadded:: 0.5
    """
    def __init__(self, item, surface_manager, parent, default_icon = None):
        """
        Default constructor
        """
        super(ItemGlyph, self).__init__(parent)

        self.item = item
        self.surface_manager = surface_manager
        self.default_icon = default_icon

        self.grid_layout = None
        self.display = None
        self.icon = None

        self.setFocusPolicy(Qt.StrongFocus)

        self.__set_layout()

    ItemFocused = pyqtSignal(Item, name='ItemFocused')
    ItemActionA = pyqtSignal(Item, name='ItemActionA')
    ItemActionB = pyqtSignal(Item, name='ItemActionB')
    ItemActionX = pyqtSignal(Item, name='ItemActionX')
    ItemActionY = pyqtSignal(Item, name='ItemActionY')

    def __set_layout(self):
        """
        Set layout of this widget
        """
        self.grid_layout = QGridLayout()
        self.grid_layout.setSpacing(0)
        self.display = QLabel()

        self.set_item(self.item)

        self.display.setMaximumSize(40, 40)
        self.display.setObjectName('passive_inventorybox')

        self.grid_layout.addWidget(self.display)
        self.setLayout(self.grid_layout)

    def focusInEvent(self, event): #pylint: disable-msg=C0103
        """
        Handle focus in

        .. versionadded:: 0.6
        """
        self.display.setObjectName('active_inventorybox')
        self.display.setStyle(QApplication.style())
        if self.item != None:
            self.ItemFocused.emit(self.item)

    def focusOutEvent(self, event):
        """
        Handle focus out

        .. versionadded:: 0.6
        """
        self.display.setObjectName('passive_inventorybox')
        self.display.setStyle(QApplication.style())

    def enabled(self, enabled):
        """
        Set this control enabled or disabled

        .. versionadded:: 0.6
        """
        if enabled == True:
            self.setFocusPolicy(Qt.StrongFocus)
        else:
            self.setFocusPolicy(Qt.NoFocus)

    def set_item(self, item):
        """
        Set item to be displayed

        .. versionadded:: 0.6
        """
        self.item = item

        if item != None:
            if item.ammunition_data != None:
                count = item.ammunition_data.count
            else:
                count = 1

            icon = self.surface_manager.get_icon(item.icon)

            if count != 1:
                image = icon.toImage()
                painter = QPainter(image)
                painter.setPen(Qt.white)
                font = QFont('Helvetica', 180, QFont.Bold)
                painter.setFont(font)
                painter.drawText(image.rect(),
                                 Qt.AlignBottom | Qt.AlignRight,
                                 str(count))
                icon = icon.fromImage(image)
                painter = None

            self.icon = icon
        else:
            if self.default_icon == None:
                self.icon = QPixmap(':transparent.png')
            else:
                self.icon = self.default_icon

        self.display.setPixmap(self.icon)



# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
module for configuring player characters
"""
import datetime

from pyherc.data import SpecialTime
from pyherc.generators import creature_config, inventory_config
from pyherc.rules.calendar import get_special_events


def init_players(context):
    """
    Initialise creatures

    :returns: list of creature configurations
    :rtype: [CreatureConfiguration]
    """
    config = []
    surface_manager = context.surface_manager

    adventurer_f0 = surface_manager.add_icon('adventurer_f0', ':pc_adventurer_f0.png', '@', ['white', 'bold'])
    adventurer_f1 = surface_manager.add_icon('adventurer_f1', ':pc_adventurer_f1.png', '@', ['white', 'bold'])
    config.append(creature_config(name = 'Adventurer',
                                  body = 6,
                                  finesse = 7,
                                  mind = 8,
                                  hp = 9,
                                  speed = 2.5,
                                  icons = (adventurer_f0, adventurer_f1),
                                  attack = 1,
                                  ai = None,
                                  effect_handles = None,
                                  inventory = [inventory_config(
                                      item_name = 'sword',
                                      min_amount = 1,
                                      max_amount = 1,
                                      probability = 100),
                                               inventory_config(
                                                   item_name = 'leather armour',
                                                   min_amount = 1,
                                                   max_amount = 1,
                                                   probability = 100),
                                               inventory_config(
                                                   item_name = 'bow',
                                                   min_amount = 1,
                                                   max_amount = 1,
                                                   probability = 100),
                                               inventory_config(
                                                   item_name = 'arrow',
                                                   min_amount = 1,
                                                   max_amount = 1,
                                                   probability = 100),
                                               inventory_config(
                                                   item_name = 'war arrow',
                                                   min_amount = 1,
                                                   max_amount = 1,
                                                   probability = 100),
                                               inventory_config(
                                                   item_name = 'blunt arrow',
                                                   min_amount = 1,
                                                   max_amount = 1,
                                                   probability = 100),
                                               inventory_config(
                                                   item_name = 'healing potion',
                                                   min_amount = 1,
                                                   max_amount = 2,
                                                   probability = 50)],
                                  description = '\n'.join(['A skillful adventurer.',
                                                           '',
                                                           'Adventurer is armed and ready to explore any dungeon he sees. He is strong enough to survive combat with some of the dangers, while some he definitely should avoid',
                                                           'Adventurer also carries some potions that will help him on his journey.'])))

    warrior_f0 = surface_manager.add_icon('warrior_f0', ':pc_warrior_f0.png', '@', ['white', 'bold'])
    warrior_f1 = surface_manager.add_icon('warrior_f1', ':pc_warrior_f1.png', '@', ['white', 'bold'])
    config.append(creature_config(name = 'Warrior',
                                  body = 8,
                                  finesse = 7,
                                  mind = 6,
                                  hp = 12,
                                  speed = 2.5,
                                  icons = (warrior_f0, warrior_f1),
                                  attack = 2,
                                  ai = None,
                                  effect_handles = None,
                                  inventory = [inventory_config(
                                      item_name = 'sword',
                                      min_amount = 1,
                                      max_amount = 1,
                                      probability = 100),
                                               inventory_config(
                                                   item_name = 'warhammer',
                                                   min_amount = 1,
                                                   max_amount = 1,
                                                   probability = 100),
                                               inventory_config(
                                                   item_name = 'scale mail',
                                                   min_amount = 1,
                                                   max_amount = 1,
                                                   probability = 100),
                                               inventory_config(
                                                   item_name = 'dagger',
                                                   min_amount = 1,
                                                   max_amount = 1,
                                                   probability = 100)],
                                  description = '\n'.join(['A stout warrior',
                                                                '',
                                                                'Warrior is armed to teeth and tends to solve his problems with brute force.',
                                                                'Warrior has nice selection of weapons to use but very little of anything else.'])))

    mage_f0 = surface_manager.add_icon('mage_f0', ':pc_mage_f0.png', '@', ['white', 'bold'])
    mage_f1 = surface_manager.add_icon('mage_f1', ':pc_mage_f1.png', '@', ['white', 'bold'])
    config.append(creature_config(name = 'Mage',
                                  body = 3,
                                  finesse = 5,
                                  mind = 11,
                                  hp = 6,
                                  speed = 2.5,
                                  icons = (mage_f0, mage_f1),
                                  attack = 1,
                                  ai = None,
                                  effect_handles = None,
                                  inventory = [inventory_config(
                                      item_name = 'dagger',
                                      min_amount = 1,
                                      max_amount = 1,
                                      probability = 100),
                                               inventory_config(
                                                   item_name = 'robes',
                                                   min_amount = 1,
                                                   max_amount = 1,
                                                   probability = 100),
                                               inventory_config(
                                                   item_name = 'healing potion',
                                                   min_amount = 1,
                                                   max_amount = 2,
                                                   probability = 50)],
                                        description = '\n'.join(['A wise mage.',
                                                                '',
                                                                'A mage is physically weak and mentally strong. He should avoid combat at all cost and deal with the enemies by using his spells.',
                                                                'Mage also carries some potions that will help him on his journey.'])))

    date = datetime.date.today()
    events = get_special_events(date.year, date.month, date.day)

    if SpecialTime.christmas in events:
        for character in config:
            character.inventory.append(inventory_config(item_name = 'idol of snowman',
                                                              min_amount = 1,
                                                              max_amount = 1,
                                                              probability = 100))

    return config



# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for baseclass of every Effect
"""
from pyherc.aspects import log_debug, log_info
from pyherc.events import Event


class Effect():
    """
    Class representing effects
    """
    @log_debug
    def __init__(self, duration, frequency, tick,
                 icon, title, description):
        """
        Default constructor

        :param duration: duration of the effect in ticks
        :type duration: int
        :param frequency: frequency of the effect in ticks
        :type frequency: int
        :param tick: initial value for timer
        :type tick: int
        """
        super().__init__()
        self.duration = duration
        self.frequency = frequency
        self.tick = tick
        self.effect_name = 'effect'
        self.icon = icon
        self.title = title
        self.description = description
        self.multiple_allowed = False

    @log_info
    def trigger(self, dying_rules):
        """
        Trigger the effect

        .. Note:: This method will be called when tick reaches zero
        """
        self.do_trigger(dying_rules)
        self.post_trigger()

    @log_debug
    def do_trigger(self, dying_rules):
        """
        Override this method to contain logic of the effect
        """
        pass

    @log_debug
    def post_trigger(self):
        """
        Do house keeping after effect has been triggered
        """
        if self.duration is not None:
            self.tick = self.frequency
            self.duration = self.duration - self.frequency

    @log_debug
    def get_add_event(self):
        """
        Get event describing adding of this effect

        :returns: event describing adding of this effect
        :rtype: Event
        """
        return Event(event_type='add event',
                     level=None,
                     location=None,
                     affected_tiles=[])

    @log_debug
    def get_removal_event(self):
        """
        Get event describing removal of this event

        :return: event describing removal of this event
        :rtype: Event
        """
        return Event(event_type='remove event',
                     level=None,
                     location=None,
                     affected_tiles=[])


class EffectHandle():
    """
    Handle that can be used to construct effects
    """
    @log_debug
    def __init__(self, trigger, effect, parameters, charges):
        """
        Default constructor

        :param trigger: name of the trigger of the effect
        :type trigger: string
        :param effect: name of the effect
        :type effect: string
        :param parameters: overriding parameters for the effect
        :type parameters: {string, object}
        :param charges: amount of charges effect has
        :type charges: int
        """

        self.trigger = trigger
        self.effect = effect
        self.parameters = parameters
        self.charges = charges

    def __str__(self):
        """
        String representation of this object
        """
        return "trigger: {0}, effect: {1}, parameters: {2}, charges: {3}".format(  # noqa
            self.trigger, self.effect, self.parameters, self.charges)



# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for adding portals
"""

from pyherc.aspects import log_debug, log_info
from pyherc.data import Portal


class PortalAdderConfiguration():
    """
    Configuration class for adding portals
    """
    @log_debug
    def __init__(self, icons, level_type, location_type, chance,
                 new_level, unique, escape_stairs=False):
        """
        Default constructor

        :param icons: pair of icons to use for poratl and other end
        :param level_type: type of level this portal can be added
        :param location_type: type of location to add portal
        :param chance: chance of portal being added 1 - 100
        :param new_level: name of new level
        :param unique: is more than one instance allowed
        :param escape_stairs: are there stairs leading out of the dungeon
        """
        self.icons = icons
        self.level_type = level_type
        self.location_type = location_type
        self.chance = chance
        self.new_level = new_level
        self.is_unique = unique
        self.is_escape_stairs = escape_stairs


class PortalAdderFactory():
    """
    Class for creating portal adders
    """
    @log_debug
    def __init__(self, config, rng):
        """
        Default constructor

        :param config: configuration to use
        :type config: [PortalAdderConfiguration]
        :param rng: random number generator
        :type rng: Random
        """
        super().__init__()
        self.config = config
        self.level_generator_factory = None
        self.rng = rng

    @log_info
    def create_portal_adders(self, level_type):
        """
        Create portal adders for level type

        :param level_type: type of level to create portal adders for
        :type level_type: [string]
        :returns: generated portal adders
        :rtype: [PortalAdder]
        """
        adders = []
        matches = [x for x in self.config
                   if x.level_type == level_type
                   and self.rng.randint(0, 100) <= x.chance]

        for spec in matches:
            new_adder = PortalAdder(spec.icons,
                                    spec.location_type,
                                    spec.new_level,
                                    spec.is_escape_stairs,
                                    self.rng)
            if spec.is_unique:
                self.config.remove(spec)
            adders.append(new_adder)

        return adders


class PortalAdder():
    """
    Basic class for adding portals
    """
    @log_debug
    def __init__(self, icons,  location_type, level_generator_name,
                 escape_stairs, rng):
        """
        Default constructor

        :param icons: pair of icons to use this portal and other end
        :param location_type: type of location to add portal
        :param level_generator_name: name of level generator
        :param rng: Randon number generator
        """
        super().__init__()
        self.location_type = location_type
        self.level_generator_name = level_generator_name
        self.rng = rng
        self.icons = icons
        self.escape_stairs = escape_stairs

    @log_info
    def add_portal(self, level):
        """
        Add given stairs to the level

        :param level: level to modify
        :type level: Level
        """
        locations = level.get_locations_by_type(self.location_type)

        if len(locations) > 0:
            location = self.rng.choice(locations)
            portal = Portal(icons=self.icons,
                            level_generator_name=self.level_generator_name)
            portal.exits_dungeon = self.escape_stairs
            level.add_portal(portal, location)



# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Module for various helpers
"""
from functools import partial
from math import sqrt

from pyherc.aspects import log_debug


@log_debug
def get_target_in_direction(level, location, direction, attack_range=100):
    """
    Get target of the attack

    :param level: level to operate
    :type level: Level
    :param location: start location
    :type location: (int, int)
    :param direction: direction to follow
    :type direction: int
    :returns: target character if found, otherwise None
    :rtype: Character
    """
    target = None
    target_location = location
    target_data = None

    if direction == 9:
        return TargetData('void',
                          None,
                          None,
                          None)

    off_sets = [(0, 0),
                (0, -1), (1, -1), (1, 0), (1, 1),
                (0, 1), (-1, 1), (-1, 0), (-1, -1)]

    while (target is None
           and distance_between(location, target_location) <= attack_range):
        target_location = tuple([x for x in
                                 map(sum,
                                     zip(target_location,
                                         off_sets[direction]))])

        if level.blocks_movement(target_location[0], target_location[1]):
            target_data = TargetData('wall',
                                     target_location,
                                     None,
                                     target_data)
            target = target_data
        else:
            target = level.get_creature_at(target_location)
            if target:
                target_data = TargetData('character',
                                         target_location,
                                         target,
                                         target_data)
                target = target_data
            else:
                target_data = TargetData('void',
                                         target_location,
                                         None,
                                         target_data)

    return target_data

get_adjacent_target_in_direction = partial(get_target_in_direction,
                                           attack_range=1.5)


@log_debug
def distance_between(location1, location2):
    """
    calculate distance between two points
    """
    x_difference = location2[0] - location1[0]
    y_difference = location2[1] - location1[1]

    return sqrt(x_difference**2 + y_difference**2)


def find_direction(start, end):
    """
    Find direction from start to end
    """
    if start[0] == end[0]:
        if start[1] < end[1]:
            return 5
        else:
            return 1
    elif start[1] == end[1]:
        if start[0] < end[0]:
            return 3
        else:
            return 7


class TargetData():
    """
    Represents target

    .. versionadded:: 0.10
    """
    @log_debug
    def __init__(self, target_type, location, target, previous_target):
        """
        Default constructor
        """
        super().__init__()
        self.target_type = target_type
        self.location = location
        self.target = target
        self.previous_target = previous_target



# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
module for configuring upper crypt
"""
from herculeum.ai import FlockingHerbivore
from pyherc.config.dsl import LevelConfiguration, LevelContext
from pyherc.data.effects import EffectHandle
from pyherc.data.traps import PitTrap
from pyherc.generators import creature_config
from pyherc.generators.level.creatures import (CreatureAdder,
                                               CreatureAdderConfiguration)
from pyherc.generators.level.decorator import (AggregateDecorator,
                                               AggregateDecoratorConfig,
                                               DirectionalWallDecorator,
                                               DirectionalWallDecoratorConfig,
                                               FloorBuilderDecorator,
                                               FloorBuilderDecoratorConfig,
                                               ReplacingDecorator,
                                               ReplacingDecoratorConfig,
                                               WallBuilderDecorator,
                                               WallBuilderDecoratorConfig,
                                               WallOrnamentDecorator,
                                               WallOrnamentDecoratorConfig)
from pyherc.generators.level.items import ItemAdder, ItemAdderConfiguration
from pyherc.generators.level.partitioners import GridPartitioner
from pyherc.generators.level.portals import PortalAdderConfiguration
from pyherc.generators.level.room import (PillarRoomGenerator, PitRoomGenerator,
                                          SquareRoomGenerator)


def init_level(rng, item_generator, creature_generator, level_size, context):
    """
    Initialise upper crypt levels

    :returns: level configuration
    :rtype: LevelConfiguration
    """
    surface_manager = context.surface_manager

    floor_natural = 'natural floor'
    floor_rock = surface_manager.add_icon('floor_rock', ':rock_floor.png', '.')
    floor_constructed = 'constructed floor'
    floor_tiled = surface_manager.add_icon('floor_tiled', ':tiled_floor.png', '.')

    wall_empty = surface_manager.add_icon('empty wall', ':transparent.png', None)
    wall_natural = 'natural wall'
    wall_constructed = 'constructed wall'
    wall_ground = surface_manager.add_icon('wall_ground', ':ground_wall.png', ' ')
    wall_rock = surface_manager.add_icon('wall_rock', ':rock_wall.png', '#')
    pillar = surface_manager.add_icon('pillar', ':pillar.png', '#')

    stairs_down = surface_manager.add_icon('stairs_down', ':stairs_down.png', '>')
    stairs_up = surface_manager.add_icon('stairs_up', ':stairs_up.png', '<')

    wall = surface_manager.add_icon('crypt_wall', ':crypt_wall.png', '#')
    wall_15 = surface_manager.add_icon('crypt_wall_15', ':crypt_wall_15.png', '#')
    wall_57 = surface_manager.add_icon('crypt_wall_57', ':crypt_wall_57.png', '#')
    wall_35 = surface_manager.add_icon('crypt_wall_35', ':crypt_wall_35.png', '#')
    wall_37 = surface_manager.add_icon('crypt_wall_37', ':crypt_wall_37.png', '#')
    wall_17 = surface_manager.add_icon('crypt_wall_17', ':crypt_wall_17.png', '#')
    wall_13 = surface_manager.add_icon('crypt_wall_13', ':crypt_wall_13.png', '#')
    wall_135 = surface_manager.add_icon('crypt_wall_135', ':crypt_wall_135.png', '#')
    wall_357 = surface_manager.add_icon('crypt_wall_357', ':crypt_wall_357.png', '#')
    wall_1357 = surface_manager.add_icon('crypt_wall_1357', ':crypt_wall_1357.png', '#')
    wall_137 = surface_manager.add_icon('crypt_wall_137', ':crypt_wall_137.png', '#')
    wall_157 = surface_manager.add_icon('crypt_wall_157', ':crypt_wall_157.png', '#')

    pit_tile = 'pit'

    room_generators = [SquareRoomGenerator(floor_natural,
                                           wall_empty,
                                           floor_natural,
                                           ['upper crypt']),
                       SquareRoomGenerator(floor_constructed,
                                           wall_empty,
                                           floor_natural,
                                           ['upper crypt']),
                       PillarRoomGenerator(floor_tile = floor_constructed,
                                           corridor_tile = floor_natural,
                                           empty_tile = wall_empty,
                                           pillar_tile = wall,
                                           level_types = ['upper crypt']),
                       PillarRoomGenerator(floor_tile = floor_natural,
                                           corridor_tile = floor_natural,
                                           empty_tile = wall_empty,
                                           pillar_tile = wall,
                                           level_types = ['upper crypt']),
                       PitRoomGenerator(floor_tile = floor_natural,
                                        corridor_tile = floor_natural,
                                        empty_tile = wall_empty,
                                        pit_tile = pit_tile,
                                        trap_type=PitTrap,
                                        level_types = ['upper crypt'])
                                        ]

    level_partitioners = [GridPartitioner(['upper crypt'],
                                          4,
                                          2,
                                          rng)]

    replacer_config = ReplacingDecoratorConfig(['upper crypt'],
                                    {floor_natural: floor_rock,
                                    floor_constructed: floor_tiled},
                                    {wall_natural: wall_ground,
                                    wall_constructed: wall_rock})
    replacer = ReplacingDecorator(replacer_config)

    wallbuilder_config = WallBuilderDecoratorConfig(['upper crypt'],
                                        {wall_natural: wall_constructed},
                                        wall_empty)
    wallbuilder = WallBuilderDecorator(wallbuilder_config)

    wall_direction_config = DirectionalWallDecoratorConfig(['upper crypt'],
                                                   east_west = wall_37,
                                                   east_north = wall_13,
                                                   east_south = wall_35,
                                                   west_north = wall_17,
                                                   west_south = wall_57,
                                                   north_south = wall_15,
                                                   east_west_north = wall_137,
                                                   east_west_south = wall_357,
                                                   east_north_south = wall_135,
                                                   west_north_south = wall_157,
                                                   four_way = wall_1357,
                                                   wall = wall_constructed)

    wall_direction_builder = DirectionalWallDecorator(wall_direction_config)

    floor = surface_manager.add_icon('crypt_floor', ':crypt_floor.png', ' ')
    floor1 = surface_manager.add_icon('crypt_floor_1', ':crypt_floor_1.png', ' ')
    floor3 = surface_manager.add_icon('crypt_floor_3', ':crypt_floor_3.png', ' ')
    floor5 = surface_manager.add_icon('crypt_floor_5', ':crypt_floor_5.png', ' ')
    floor7 = surface_manager.add_icon('crypt_floor_7', ':crypt_floor_7.png', ' ')
    floor13 = surface_manager.add_icon('crypt_floor_13', ':crypt_floor_13.png', ' ')
    floor15 = surface_manager.add_icon('crypt_floor_15', ':crypt_floor_15.png', ' ')
    floor17 = surface_manager.add_icon('crypt_floor_17', ':crypt_floor_17.png', ' ')
    floor35 = surface_manager.add_icon('crypt_floor_35', ':crypt_floor_35.png', ' ')
    floor37 = surface_manager.add_icon('crypt_floor_37', ':crypt_floor_37.png', ' ')
    floor57 = surface_manager.add_icon('crypt_floor_57', ':crypt_floor_57.png', ' ')
    floor135 = surface_manager.add_icon('crypt_floor_135', ':crypt_floor_135.png', ' ')
    floor137 = surface_manager.add_icon('crypt_floor_137', ':crypt_floor_137.png', ' ')
    floor157 = surface_manager.add_icon('crypt_floor_157', ':crypt_floor_157.png', ' ')
    floor357 = surface_manager.add_icon('crypt_floor_357', ':crypt_floor_357.png', ' ')
    floor1357 = surface_manager.add_icon('crypt_floor_1357', ':crypt_floor_1357.png', ' ')

    floor_config = FloorBuilderDecoratorConfig([],
                                               single = floor,
                                               north = floor1,
                                               east = floor3,
                                               south = floor5,
                                               west = floor7,
                                               north_east = floor13,
                                               north_south = floor15,
                                               north_west = floor17,
                                               east_south = floor35,
                                               east_west = floor37,
                                               south_west = floor57,
                                               north_east_south = floor135,
                                               north_east_west = floor137,
                                               north_south_west = floor157,
                                               east_south_west = floor357,
                                               fourway = floor1357,
                                               floor = floor_natural)
    floor_builder = FloorBuilderDecorator(floor_config)

    board_floor = surface_manager.add_icon('crypt_floor_2', ':crypt_floor_2.png', ' ')
    board_floor1 = surface_manager.add_icon('crypt_floor_2_1', ':crypt_floor_2_1.png', ' ')
    board_floor3 = surface_manager.add_icon('crypt_floor_2_3', ':crypt_floor_2_3.png', ' ')
    board_floor5 = surface_manager.add_icon('crypt_floor_2_5', ':crypt_floor_2_5.png', ' ')
    board_floor7 = surface_manager.add_icon('crypt_floor_2_7', ':crypt_floor_2_7.png', ' ')
    board_floor13 = surface_manager.add_icon('crypt_floor_2_13', ':crypt_floor_2_13.png', ' ')
    board_floor15 = surface_manager.add_icon('crypt_floor_2_15', ':crypt_floor_2_15.png', ' ')
    board_floor17 = surface_manager.add_icon('crypt_floor_2_17', ':crypt_floor_2_17.png', ' ')
    board_floor35 = surface_manager.add_icon('crypt_floor_2_35', ':crypt_floor_2_35.png', ' ')
    board_floor37 = surface_manager.add_icon('crypt_floor_2_37', ':crypt_floor_2_37.png', ' ')
    board_floor57 = surface_manager.add_icon('crypt_floor_2_57', ':crypt_floor_2_57.png', ' ')
    board_floor135 = surface_manager.add_icon('crypt_floor_2_135', ':crypt_floor_2_135.png', ' ')
    board_floor137 = surface_manager.add_icon('crypt_floor_2_137', ':crypt_floor_2_137.png', ' ')
    board_floor157 = surface_manager.add_icon('crypt_floor_2_157', ':crypt_floor_2_157.png', ' ')
    board_floor357 = surface_manager.add_icon('crypt_floor_2_357', ':crypt_floor_2_357.png', ' ')
    board_floor1357 = surface_manager.add_icon('crypt_floor_2_1357', ':crypt_floor_2_1357.png', ' ')

    board_floor_config = FloorBuilderDecoratorConfig([],
                                               single = board_floor,
                                               north = board_floor1,
                                               east = board_floor3,
                                               south = board_floor5,
                                               west = board_floor7,
                                               north_east = board_floor13,
                                               north_south = board_floor15,
                                               north_west = board_floor17,
                                               east_south = board_floor35,
                                               east_west = board_floor37,
                                               south_west = board_floor57,
                                               north_east_south = board_floor135,
                                               north_east_west = board_floor137,
                                               north_south_west = board_floor157,
                                               east_south_west = board_floor357,
                                               fourway = board_floor1357,
                                               floor = floor_constructed)
    board_floor_builder = FloorBuilderDecorator(board_floor_config)

    pit = surface_manager.add_icon('brick_pit_07', ':brick_pit_07.png', '^')
    pit1 = surface_manager.add_icon('brick_pit_08', ':brick_pit_08.png', '^')
    pit3 = surface_manager.add_icon('brick_pit_01', ':brick_pit_01.png', '^')
    pit5 = surface_manager.add_icon('brick_pit_07', ':brick_pit_07.png', '^')
    pit7 = surface_manager.add_icon('brick_pit_03', ':brick_pit_03.png', '^')
    pit13 = surface_manager.add_icon('brick_pit_04', ':brick_pit_04.png', '^')
    pit15 = surface_manager.add_icon('brick_pit_08', ':brick_pit_08.png', '^')
    pit17 = surface_manager.add_icon('brick_pit_06', ':brick_pit_06.png', '^')
    pit35 = surface_manager.add_icon('brick_pit_01', ':brick_pit_01.png', '^')
    pit37 = surface_manager.add_icon('brick_pit_02', ':brick_pit_02.png', '^')
    pit57 = surface_manager.add_icon('brick_pit_03', ':brick_pit_03.png', '^')
    pit135 = surface_manager.add_icon('brick_pit_04', ':brick_pit_04.png', '^')
    pit137 = surface_manager.add_icon('brick_pit_05', ':brick_pit_05.png', '^')
    pit157 = surface_manager.add_icon('brick_pit_06', ':brick_pit_06.png', '^')
    pit357 = surface_manager.add_icon('brick_pit_02', ':brick_pit_02.png', '^')
    pit1357 = surface_manager.add_icon('brick_pit_05', ':brick_pit_05.png', '^')

    pit_config = FloorBuilderDecoratorConfig([],
                                             single = pit,
                                             north = pit1,
                                             east = pit3,
                                             south = pit5,
                                             west = pit7,
                                             north_east = pit13,
                                             north_south = pit15,
                                             north_west = pit17,
                                             east_south = pit35,
                                             east_west = pit37,
                                             south_west = pit57,
                                             north_east_south = pit135,
                                             north_east_west = pit137,
                                             north_south_west = pit157,
                                             east_south_west = pit357,
                                             fourway = pit1357,
                                             floor = pit_tile)
    pit_builder = FloorBuilderDecorator(pit_config)

    torches_tile_f0 = surface_manager.add_icon('crypt_torches_f0', ':wall_torches_f0.png', '¤')
    torches_tile_f1 = surface_manager.add_icon('crypt_torches_f1', ':wall_torches_f1.png', '¤')
    torch_tile_f0 = surface_manager.add_icon('crypt_torch_f0', ':wall_torch_f0.png', '¤')
    torch_tile_f1 = surface_manager.add_icon('crypt_torch_f1', ':wall_torch_f1.png', '¤')

    torch_ornamenter_config = WallOrnamentDecoratorConfig(
                                                ['upper crypt'],
                                                wall_tile = wall_37,
                                                ornamentation = [(torch_tile_f0,
                                                                  torch_tile_f1),
                                                                 (torches_tile_f0,
                                                                  torches_tile_f1)],
                                                rng = rng,
                                                rate = 13)
    torch_ornamenter = WallOrnamentDecorator(torch_ornamenter_config)

    aggregate_decorator_config = AggregateDecoratorConfig(['upper crypt'],
                                                          [wallbuilder,
                                                           wall_direction_builder,
                                                           floor_builder,
                                                           board_floor_builder,
                                                           pit_builder,
                                                           torch_ornamenter,
                                                           replacer])

    decorators = [AggregateDecorator(aggregate_decorator_config)]

    item_adder_config = ItemAdderConfiguration(['upper crypt'])
    item_adder_config.add_item(min_amount = 2,
                               max_amount = 4,
                               type = 'weapon',
                               location = 'room')
    item_adder_config.add_item(min_amount = 2,
                               max_amount = 4,
                               type = 'potion',
                               location = 'room')
    item_adder_config.add_item(min_amount = 0,
                               max_amount = 5,
                               type = 'food',
                               location = 'room')
    item_adder_config.add_item(min_amount = 0,
                               max_amount = 1,
                               type = 'tome',
                               location = 'room')
    item_adders = [ItemAdder(item_generator,
                            item_adder_config,
                            rng)]

    creature_adder_config = CreatureAdderConfiguration(['upper crypt'])

    creature_adder_config.add_creature(min_amount = 4,
                                       max_amount = 8,
                                       name = 'spider')

    creature_adders = [CreatureAdder(creature_generator,
                                    creature_adder_config,
                                    rng)]

    portal_adder_configurations = [PortalAdderConfiguration(
                                        icons = (stairs_down,
                                                 stairs_up),
                                        level_type = 'lower catacombs',
                                        location_type = 'room',
                                        chance = 10,
                                        new_level = 'upper crypt',
                                        unique = True)]

    level_context = LevelContext(size = level_size,
                                 floor_type = None,
                                 wall_type = wall_natural,
                                 empty_floor = 0,
                                 empty_wall = wall_empty,
                                 level_types = ['upper crypt',
                                                'lower crypt'])

    config = (LevelConfiguration()
                    .with_rooms(room_generators)
                    .with_partitioners(level_partitioners)
                    .with_decorators(decorators)
                    .with_items(item_adders)
                    .with_creatures(creature_adders)
                    .with_portals(portal_adder_configurations)
                    .with_contexts([level_context])
                    .build())
    return config

def init_creatures(context):
    """
    Initialise creatures

    :returns: list of creature configurations
    :rtype: [CreatureConfiguration]
    """
    config = []
    surface_manager = context.surface_manager

    spider_f0 = surface_manager.add_icon('spider_f0', ':masked-spider_f0.png', 's', ['white', 'dim'])
    spider_f1 = surface_manager.add_icon('spider_f1', ':masked-spider_f1.png', 's', ['white', 'dim'])
    config.append(creature_config(name = 'spider',
                                  body = 6,
                                  finesse = 12,
                                  mind = 8,
                                  hp = 6,
                                  speed = 1,
                                  icons = (spider_f0, spider_f1),
                                  attack = 4,
                                  ai = FlockingHerbivore,
                                  effect_handles = [EffectHandle(
                                      trigger = 'on attack hit',
                                      effect = 'minor poison',
                                      parameters = None,
                                      charges = 100)]))

    return config



# -*- coding: utf-8 -*-

#   Copyright 2010-2014 Tuukka Turto
#
#   This file is part of pyherc.
#
#   pyherc is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   pyherc is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with pyherc.  If not, see <http://www.gnu.org/licenses/>.

"""
Classes for poisoning
"""
from .animation import Animation
from .counters import DamageCounter
from random import Random
from PyQt4.QtCore import (QEasingCurve, QPropertyAnimation,
                          QSequentialAnimationGroup)
from herculeum.ui.gui.layers import zorder_counter


class PoisonAddedAnimation(Animation):
    """
    Animation for adding poison

    .. versionadded:: 0.12
    """
    def __init__(self, event):
        """
        Default constructor
        """
        super().__init__(event)

        self.location = event.target.location
        self.text = 'poisoned'
        self.colour = 'green'
        self.offset = (0, 16)

    def trigger(self, ui):
        """
        Trigger this animation
        """
        damage_counter = DamageCounter(damage=self.text,
                                       colour=self.colour,
                                       parent=ui)
        ui.view.scene().addItem(damage_counter)
        damage_counter.setZValue(zorder_counter)

        bounds = damage_counter.boundingRect()
        width = bounds.width()

        damage_counter.setPos(self.location[0] * 32 + 16
                              - (width / 2) + self.offset[0],
                              self.location[1] * 32 + self.offset[1])

        animation = QSequentialAnimationGroup()

        moving = QPropertyAnimation(damage_counter.adapter,
                                    'y_location')
        moving.setDuration(750)
        moving.setStartValue(self.location[1] * 32)
        moving.setEndValue(self.location[1] * 32 - 32)

        animation.addAnimation(moving)

        fading = QPropertyAnimation(damage_counter.adapter,
                                    'opacity')
        fading.setDuration(750)
        fading.setStartValue(1.0)
        fading.setEndValue(0.0)
        animation.addAnimation(fading)

        animation.finished.connect(ui.remove_finished_animation)
        ui.animations.append(animation)

        animation.start()


class PoisonTriggeredAnimation(Animation):
    """
    Animation for poison triggering

    .. versionadded:: 0.12
    """
    def __init__(self, event):
        """
        Default constructor
        """
        super().__init__(event)

        self.location = event.target.location
        self.damage = -event.damage
        self.colour = 'green'
        self.offset = (0, 16)

    def trigger(self, ui):
        """
        Trigger this animation
        """
        damage_counter = DamageCounter(damage=str(self.damage),
                                       colour=self.colour,
                                       parent=ui)
        ui.view.scene().addItem(damage_counter)
        damage_counter.setZValue(zorder_counter)

        bounds = damage_counter.boundingRect()
        width = bounds.width()

        rand = Random()

        damage_counter.setPos((self.location[0] * 32
                               + 16 - (width / 2)
                               + rand.randint(-16, 16)) + self.offset[0],
                              self.location[1] * 32 + self.offset[1])

        animation = QSequentialAnimationGroup()

        moving = QPropertyAnimation(damage_counter.adapter,
                                    'y_location')
        moving.setDuration(750)
        moving.setStartValue(self.location[1] * 32 + self.offset[1])
        moving.setEndValue(self.location[1] * 32 - 32 + self.offset[1])
        curve = QEasingCurve(QEasingCurve.OutElastic)
        moving.setEasingCurve(curve)
        animation.addAnimation(moving)

        fading = QPropertyAnimation(damage_counter.adapter,
                                    'opacity')
        fading.setDuration(750)
        fading.setStartValue(1.0)
        fading.setEndValue(0.0)
        animation.addAnimation(fading)

        animation.finished.connect(ui.remove_finished_animation)
        ui.animations.append(animation)

        animation.start()
